# cython: binding=True, language_level=3
# mode: run
# tag: cyfunction

import inspect

try:
    sig = inspect.Signature.from_callable
except AttributeError:
    sig = inspect.Signature.from_function


def signatures_match(f1, f2):
    if sig(f1) == sig(f2):
        return None  # nothing to show in doctest
    return sig(f1), sig(f2)


lb = lambda a, b, c: None

def b(a, b, c):
    """
    >>> def py_b(a, b, c): pass
    >>> signatures_match(b, py_b)

    >>> py_lb = lambda a, b, c: None
    >>> signatures_match(lb, py_lb)
    """


lc = lambda a, b, c=1: None

def c(a, b, c=1):
    """
    >>> def py_c(a, b, c=1): pass
    >>> signatures_match(c, py_c)

    >>> py_lc = lambda a, b, c=1: None
    >>> signatures_match(lc, py_lc)
    """


ld = lambda a, b, *, c = 88: None

def d(a, b, *, c = 88):
    """
    >>> def py_d(a, b, *, c = 88): pass
    >>> signatures_match(d, py_d)


    >>> py_ld = lambda a, b, *, c = 88: None
    >>> signatures_match(ld, py_ld)
    """


le = lambda a, b, c = 88, **kwds: None

def e(a, b, c = 88, **kwds):
    """
    >>> def py_e(a, b, c = 88, **kwds): pass
    >>> signatures_match(e, py_e)

    >>> py_le = lambda a, b, c = 88, **kwds: None
    >>> signatures_match(le, py_le)
    """


lf = lambda a, b, *, c, d = 42: None

def f(a, b, *, c, d = 42):
    """
    >>> def py_f(a, b, *, c, d = 42): pass
    >>> signatures_match(f, py_f)

    >>> py_lf = lambda a, b, *, c, d = 42: None
    >>> signatures_match(lf, py_lf)
    """


lg = lambda a, b, *, c, d = 42, e = 17, f, **kwds: None

def g(a, b, *, c, d = 42, e = 17, f, **kwds):
    """
    >>> def py_g(a, b, *, c, d = 42, e = 17, f, **kwds): pass
    >>> signatures_match(g, py_g)

    >>> py_lg = lambda a, b, *, c, d = 42, e = 17, f, **kwds: None
    >>> signatures_match(lg, py_lg)
    """


lh = lambda a, b, *args, c, d = 42, e = 17, f, **kwds: None

def h(a, b, *args, c, d = 42, e = 17, f, **kwds):
    """
    >>> def py_h(a, b, *args, c, d = 42, e = 17, f, **kwds): pass
    >>> signatures_match(h, py_h)

    >>> py_lh = lambda a, b, *args, c, d = 42, e = 17, f, **kwds: None
    >>> signatures_match(lh, py_lh)
    """


lk = lambda a, b, c=1, *args, d = 42, e = 17, f, **kwds: None

def k(a, b, c=1, *args, d = 42, e = 17, f, **kwds):
    """
    >>> def py_k(a, b, c=1, *args, d = 42, e = 17, f, **kwds): pass
    >>> signatures_match(k, py_k)

    >>> py_lk = lambda a, b, c=1, *args, d = 42, e = 17, f, **kwds: None
    >>> signatures_match(lk, py_lk)
    """


ll = lambda *, a, b, c = 88: None

def l(*, a, b, c = 88):
    """
    >>> def py_l(*, a, b, c = 88): pass
    >>> signatures_match(l, py_l)

    >>> py_ll = lambda *, a, b, c = 88: None
    >>> signatures_match(ll, py_ll)
    """


lm = lambda a, *, b, c = 88: None

def m(a, *, b, c = 88):
    """
    >>> def py_m(a, *, b, c = 88): pass
    >>> signatures_match(m, py_m)

    >>> py_lm = lambda a, *, b, c = 88: None
    >>> signatures_match(lm, py_lm)
    """
    a, b, c = b, c, a


ln = lambda a, *, b, c = 88: None

def n(a, *, b, c = 88):
    """
    >>> def py_n(a, *, b, c = 88): pass
    >>> signatures_match(n, py_n)

    >>> py_ln = lambda a, *, b, c = 88: None
    >>> signatures_match(ln, py_ln)
    """


cpdef cp1(a, b):
    """
    >>> def py_cp1(a, b): pass
    >>> signatures_match(cp1, py_cp1)
    """


cpdef cp2(a, b=True):
    """
    >>> def py_cp2(a, b=True): pass

    >>> signatures_match(cp2, py_cp2)
    """


cpdef cp3(a=1, b=True):
    """
    >>> def py_cp3(a=1, b=True): pass

    >>> signatures_match(cp3, py_cp3)
    """



from cython.cimports.cython import view

def main():
    # direct access in both dimensions, strided in the first dimension, contiguous in the last
    a: cython.int[:, ::view.contiguous]

    # contiguous list of pointers to contiguous lists of ints
    b: cython.int[::view.indirect_contiguous, ::1]

    # direct or indirect in the first dimension, direct in the second dimension
    # strided in both dimensions
    c: cython.int[::view.generic, :]



# distutils: language = c++

from cpython cimport Py_buffer
from libcpp.vector cimport vector


cdef class Matrix:

    cdef int view_count

    cdef Py_ssize_t ncols
    cdef vector[float] v
    # ...

    def __cinit__(self, Py_ssize_t ncols):
        self.ncols = ncols
        self.view_count = 0

    def add_row(self):
        if self.view_count > 0:
            raise ValueError("can't add row while being viewed")
        self.v.resize(self.v.size() + self.ncols)

    def __getbuffer__(self, Py_buffer *buffer, int flags):
        # ... as before

        self.view_count += 1

    def __releasebuffer__(self, Py_buffer *buffer):
        self.view_count -= 1



cdef extern from *:
    """
    struct CrunchyType {
        int number;
        PyObject* string;
    };
    """
    cdef class crunchytype.Crunchy [ object CrunchyType ]:
        cdef int number
        cdef object string



# mode: compile
# tag: memoryview

# cython: binding=False

# See GH 3613 - when memoryviews were compiled with binding off they ended up in an
# inconsistent state where different directives were applied at different stages
# of the pipeline

import cython

def f(double[:] a):
    pass

@cython.binding(False)
def g(double[:] a):
    pass

@cython.binding(True)
def h(double[:] a):
    pass



# tag: numpy
import numpy
from cython.cimports.numpy import int32_t

def main():
    a: int32_t[:] = numpy.arange(10, dtype=numpy.int32)
    a = a[::2]

    print(a)
    print(numpy.asarray(a))
    print(a.base)

# this prints:
#    <MemoryView of 'ndarray' object>
#    [0 2 4 6 8]
#    [0 1 2 3 4 5 6 7 8 9]



import cython
from cython.cimports.libc.stdlib import free

@cython.cclass
class OwnedPointer:
    ptr: cython.p_void

    def __dealloc__(self):
        if self.ptr is not cython.NULL:
            free(self.ptr)

    @staticmethod
    @cython.cfunc
    def create(ptr: cython.p_void):
        p = OwnedPointer()
        p.ptr = ptr
        return p



#cython: c_string_type = unicode
#cython: c_string_encoding = ascii

auto_string_type = unicode

from libc.string cimport strcmp


def _as_string(x):
    try:
        return x.decode('latin1')
    except AttributeError:
        return x


def as_objects(char* ascii_data):
    """
    >>> x = as_objects('abc')
    >>> isinstance(x, auto_string_type) or type(x)
    True
    >>> _as_string(x) == 'abc' or repr(x)
    True
    """
    assert isinstance(<object>ascii_data, auto_string_type)
    assert isinstance(<bytes>ascii_data, bytes)
    assert isinstance(<str>ascii_data, str)
    assert isinstance(<unicode>ascii_data, unicode)
    return ascii_data


def charptr_as_bool(char *ascii_data):
    """
    >>> charptr_as_bool('abc')
    (True, True, True, True, True, True)
    >>> charptr_as_bool('')
    (True, False, False, False, False, False)
    """
    a = False
    if ascii_data:  # Tests the pointer, not the string.
        a = True

    b = bool(ascii_data)
    c = bool(<object> ascii_data)
    d = bool(<bytes> ascii_data)
    e = bool(<str> ascii_data)
    f = bool(<unicode> ascii_data)

    return (a, b, c, d, e, f)


def carray_as_bool(char *ascii_data):
    """
    >>> carray_as_bool('abc')
    (True, True, True, True, True)
    >>> carray_as_bool('')
    (False, False, False, False, False)
    """
    cdef char[4] array
    for i in range(4):
        array[i] = ascii_data[i]
        if not ascii_data[i]:
            break

    b = bool(array)
    c = bool(<object> array)
    d = bool(<bytes> array)
    e = bool(<str> array)
    f = bool(<unicode> array)

    return (b, c, d, e, f)


def from_object():
    """
    >>> from_object()
    """
    cdef bytes b = b"abc"
    cdef str s = "abc"
    cdef unicode u = u"abc"
    assert strcmp(<char*>b, "abc") == 0
    assert strcmp(<char*>s, "abc") == 0
    assert strcmp(<char*>u, "abc") == 0


def slice_as_objects(char* ascii_data, int start, int end):
    """
    >>> x = slice_as_objects('grok', 1, 3)
    >>> isinstance(x, auto_string_type) or type(x)
    True
    >>> _as_string(x) == 'ro' or repr(x)
    True
    """
    assert isinstance(<object>ascii_data[start:end], auto_string_type)
    assert isinstance(<bytes>ascii_data[start:end], bytes)
    assert isinstance(<str>ascii_data[start:end], str)
    assert isinstance(<unicode>ascii_data[start:end], unicode)

    assert isinstance(<object>ascii_data[start:], auto_string_type)
    assert isinstance(<bytes>ascii_data[start:], bytes)
    assert isinstance(<str>ascii_data[start:], str)
    assert isinstance(<unicode>ascii_data[start:], unicode)

    return ascii_data[start:end]



# mode: error
# cython: language_level=3

def int_literals():
    a = 1L
    b = 10000000000000L
    c = 1UL
    d = 10000000000000UL
    e = 10000000000000LL


_ERRORS = """
5:8: illegal integer literal syntax in Python source file
6:8: illegal integer literal syntax in Python source file
7:8: illegal integer literal syntax in Python source file
8:8: illegal integer literal syntax in Python source file
9:8: illegal integer literal syntax in Python source file
"""



# mode: error
# tag: assert

def nontrivial_assert_in_nogil(int a, obj):
    with nogil:
        # NOK
        assert obj
        assert a*obj
        assert obj, "abc"

        # OK
        assert a
        assert a*a
        assert a, "abc"
        assert a, u"abc"
        assert a, f"123{a}xyz"


_ERRORS = """
7:15: Truth-testing Python object not allowed without gil
8:15: Converting to Python object not allowed without gil
8:16: Operation not allowed without gil
8:16: Truth-testing Python object not allowed without gil
9:15: Truth-testing Python object not allowed without gil
"""



# distutils: language = c++
from cython.cimports.cpython import Py_buffer
from cython.cimports.libcpp.vector import vector

@cython.cclass
class Matrix:
    ncols: cython.Py_ssize_t
    shape: cython.Py_ssize_t[2]
    strides: cython.Py_ssize_t[2]
    v: vector[cython.float]

    def __cinit__(self, ncols: cython.Py_ssize_t):
        self.ncols = ncols

    def add_row(self):
        """Adds a row, initially zero-filled."""
        self.v.resize(self.v.size() + self.ncols)

    def __getbuffer__(self, buffer: cython.pointer[Py_buffer], flags: cython.int):
        itemsize: cython.Py_ssize_t = cython.sizeof(self.v[0])

        self.shape[0] = self.v.size() // self.ncols
        self.shape[1] = self.ncols

        # Stride 1 is the distance, in bytes, between two items in a row;
        # this is the distance between two adjacent items in the vector.
        # Stride 0 is the distance between the first elements of adjacent rows.
        self.strides[1] = cython.cast(cython.Py_ssize_t, (
             cython.cast(cython.p_char, cython.address(self.v[1]))
           - cython.cast(cython.p_char, cython.address(self.v[0]))
           )
       )
        self.strides[0] = self.ncols * self.strides[1]

        buffer.buf = cython.cast(cython.p_char, cython.address(self.v[0]))
        buffer.format = 'f'                     # float
        buffer.internal = cython.NULL           # see References
        buffer.itemsize = itemsize
        buffer.len = self.v.size() * itemsize   # product(shape) * itemsize
        buffer.ndim = 2
        buffer.obj = self
        buffer.readonly = 0
        buffer.shape = self.shape
        buffer.strides = self.strides
        buffer.suboffsets = cython.NULL         # for pointer arrays only

    def __releasebuffer__(self, buffer: cython.pointer[Py_buffer]):
        pass



from cython.parallel cimport prange
cimport cython
from libc.math cimport sqrt

@cython.boundscheck(False)
@cython.wraparound(False)
def l2norm(double[:] x):
    cdef double total = 0
    cdef Py_ssize_t i
    for i in prange(x.shape[0], nogil=True):
        total += x[i]*x[i]
    return sqrt(total)




def exception_creates_invalid_instance():
    """
    >>> print( exception_creates_invalid_instance() )
    OK
    """
    class MyException(Exception):
        def __new__(cls, *args):
            return object()

    try:
        raise MyException
    except TypeError:
        return "OK"



# mode: run
# ticket: t593
# tag: property, decorator

"""
>>> am_i_buggy
False
"""

def testme(func):
    try:
        am_i_buggy
        return True
    except NameError:
        return False

@testme
def am_i_buggy():
    pass

def called_deco(a,b,c):
    a.append( (1,b,c) )
    def count(f):
        a.append( (2,b,c) )
        return f
    return count

L = []

@called_deco(L, 5, c=6)
@called_deco(L, c=3, b=4)
@called_deco(L, 1, 2)
def wrapped_func(x):
    """
    >>> L
    [(1, 5, 6), (1, 4, 3), (1, 1, 2), (2, 1, 2), (2, 4, 3), (2, 5, 6)]
    >>> wrapped_func(99)
    99
    >>> L
    [(1, 5, 6), (1, 4, 3), (1, 1, 2), (2, 1, 2), (2, 4, 3), (2, 5, 6)]
    """
    return x


def class_in_closure(x):
    """
    >>> C1, c0 = class_in_closure(5)
    >>> C1().smeth1()
    (5, ())
    >>> C1.smeth1(1,2)
    (5, (1, 2))
    >>> C1.smeth1()
    (5, ())
    >>> c0.smeth0()
    1
    >>> c0.__class__.smeth0()
    1
    """
    class ClosureClass1(object):
        @staticmethod
        def smeth1(*args):
            return x, args

    class ClosureClass0(object):
        @staticmethod
        def smeth0():
            return 1

    return ClosureClass1, ClosureClass0()

def class_not_in_closure():
    """
    >>> c = class_not_in_closure()
    >>> c.smeth0()
    1
    >>> c.__class__.smeth0()
    1
    """
    class ClosureClass0(object):
        @staticmethod
        def smeth0():
            return 1

    return ClosureClass0()



# tag: numpy
# tag: openmp

cimport cython
from cython.parallel import prange
cimport numpy as np


@cython.boundscheck(False)
def test_parallel_numpy_arrays():
    """
    >>> test_parallel_numpy_arrays()
    -5
    -4
    -3
    -2
    -1
    0
    1
    2
    3
    4
    """
    cdef Py_ssize_t i, length
    cdef np.ndarray[np.int64_t] x

    try:
        import numpy
    except ImportError:
        for i in range(-5, 5):
            print i
        return

    x = numpy.zeros(10, dtype=numpy.int_)
    length = x.shape[0]

    for i in prange(length, nogil=True):
        x[i] = i - 5

    for i in x:
        print(i)



def single_py(a, b):
    """
    >>> single_py(1, 2)
    True
    >>> single_py(2, 1)
    False
    """
    return a < b

def cascaded_py(a, b, c):
    """
    >>> cascaded_py(1, 2, 3)
    True
    >>> cascaded_py(1, 2, -1)
    False
    >>> cascaded_py(10, 2, 3)
    False
    """
    return a < b < c

def single_c(int a, int b):
    """
    >>> single_c(1, 2)
    True
    >>> single_c(2, 1)
    False
    """
    return a < b

def cascaded_c(double a, double b, double c):
    """
    >>> cascaded_c(1, 2, 3)
    True
    >>> cascaded_c(1, 2, -1)
    False
    >>> cascaded_c(10, 2, 3)
    False
    """
    return a < b < c

def cascaded_mix_pyleft(a, double b, double c):
    """
    >>> cascaded_mix_pyleft(1, 2, 3)
    True
    >>> cascaded_mix_pyleft(1, 2, -1)
    False
    >>> cascaded_mix_pyleft(10, 2, 3)
    False
    """
    return a < b < c

def cascaded_mix_pyright(double a, double b, c):
    """
    >>> cascaded_mix_pyright(1, 2, 3)
    True
    >>> cascaded_mix_pyright(1, 2, -1)
    False
    >>> cascaded_mix_pyright(10, 2, 3)
    False
    """
    return a < b < c

def typed_cmp(list L):
    """
    >>> typed_cmp([1,2,3])
    False
    False
    False
    False
    """
    print L is Ellipsis
    print Ellipsis is L
    print 1 == L
    print L == 1.5

def pointer_cmp():
    """
    >>> pointer_cmp()
    True
    False
    True
    """
    cdef int* a = NULL
    cdef double* b = NULL
    cdef double** c = NULL
    print a is NULL
    print b is not NULL
    print c == NULL

def c_cmp(double a, int b, long c):
    """
    >>> c_cmp(1, 2, 3)
    True
    >>> c_cmp(1.5, 2, 2)
    True
    >>> c_cmp(1.5, 2, 0)
    False
    >>> c_cmp(1, 1, 3)
    False
    """
    return a < b <= c



# mode: run
# tag: cpp, werror, cpp17, cppexecpolicies, no-cpp-locals

from libcpp.algorithm cimport is_sorted, sort, stable_sort, nth_element, all_of, count, copy
from libcpp.execution cimport seq
from libcpp.vector cimport vector
from libcpp.functional cimport greater
from libcpp.iterator cimport back_inserter
from libcpp cimport bool


def is_sorted_ints(vector[int] values):
    """
    Test is_sorted.

    >>> is_sorted_ints([3, 1, 4, 1, 5])
    False
    >>> is_sorted_ints([1, 1, 3, 4, 5])
    True
    """
    return is_sorted(seq, values.begin(), values.end())


def sort_ints_reverse(vector[int] values):
    """Test sort using a standard library comparison function object.

    >>> sort_ints_reverse([5, 7, 4, 2, 8, 6, 1, 9, 0, 3])
    [9, 8, 7, 6, 5, 4, 3, 2, 1, 0]
    """
    sort(seq, values.begin(), values.end(), greater[int]())
    return values


def count_ones(vector[int] values):
    """
    Test count.

    >>> count_ones([1, 2, 1, 2])
    2
    """
    return count(seq, values.begin(), values.end(), 1)


def copy_int(vector[int] values):
    """
    Test copy.

    >>> copy_int(range(5))
    [0, 1, 2, 3, 4]
    """
    cdef vector[int] out
    copy(seq, values.begin(), values.end(), back_inserter(out))
    return out



# mode: run
# tag: dataclass, pure3.7

from __future__ import print_function

import cython

# cython.dataclasses implicitly implies cclass
@cython.dataclasses.dataclass(order=True, unsafe_hash=True)
class MyDataclass:
    """
    >>> sorted(list(MyDataclass.__dataclass_fields__.keys()))
    ['a', 'self']
    >>> inst1 = MyDataclass(2, ['a', 'b'])
    >>> print(inst1)
    MyDataclass(a=2, self=['a', 'b'])
    >>> inst2 = MyDataclass()
    >>> print(inst2)
    MyDataclass(a=1, self=[])
    >>> inst1 == inst2
    False
    >>> inst1 > inst2
    True
    >>> inst2 == MyDataclass()
    True
    >>> hash(inst1) != id(inst1)
    True
    >>> inst1.func_with_annotations(2.0)
    4.0
    """

    a: int = 1
    self: list = cython.dataclasses.field(default_factory=list, hash=False)  # test that arguments of init don't conflict

    def func_with_annotations(self, b: float):
        c: float = b
        return self.a * c


class DummyObj:
    def __repr__(self):
        return "DummyObj()"


@cython.dataclasses.dataclass
@cython.cclass
class NoInitFields:
    """
    >>> NoInitFields()
    NoInitFields(has_default=DummyObj(), has_factory='From a lambda', neither=None)
    >>> NoInitFields().has_default is NoInitFields().has_default
    True

    >>> NoInitFields(1)  # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    TypeError: NoInitFields.__init__() takes 1 positional argument but 2 were given

    >>> NoInitFields(has_default=1)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...has_default...
    >>> NoInitFields(has_factory=1)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...has_factory...
    >>> NoInitFields(neither=1)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...neither...
    """
    has_default : object = cython.dataclasses.field(default=DummyObj(), init=False)
    has_factory : object = cython.dataclasses.field(default_factory=lambda: "From a lambda", init=False)
    # Cython will default-initialize to None
    neither : object = cython.dataclasses.field(init=False)

    def __post_init__(self):
        if not cython.compiled:
            # Cython will default-initialize this to None, while Python won't
            # and not initializing it will mess up repr
            assert not hasattr(self, "neither")
            self.neither = None


@cython.dataclasses.dataclass
class NonInitDefaultArgument:
    """
    >>> NonInitDefaultArgument(1.0, "hello")
    NonInitDefaultArgument(x=1.0, y=10, z='hello')
    """
    x: float
    y: int = cython.dataclasses.field(default=10, init=False)
    z: str  # This is allowed despite following a default argument, because the default argument isn't in init



# cython: binding=True
# mode: run
# tag: cyfunction, closures

cimport cython


def get_defaults(func):
    return func.__defaults__


def test_defaults_none():
    """
    >>> get_defaults(test_defaults_none)
    """

def test_defaults_literal(a=1, b=(1,2,3)):
    """
    >>> get_defaults(test_defaults_literal) is get_defaults(test_defaults_literal)
    True
    >>> get_defaults(test_defaults_literal)
    (1, (1, 2, 3))
    >>> a, b = get_defaults(test_defaults_literal)
    >>> c, d = test_defaults_literal()
    >>> a is c
    True
    >>> b is d
    True
    """
    return a, b

def test_defaults_nonliteral():
    """
    >>> f0, f1 = test_defaults_nonliteral()
    >>> get_defaults(f0) is get_defaults(f0) # cached
    True
    >>> get_defaults(f0)
    (0, {}, (1, 2, 3))
    >>> a, b = get_defaults(f0)[1:]
    >>> c, d = f0(0)
    >>> a is c
    True
    >>> b is d
    True
    >>> get_defaults(f1) is get_defaults(f1) # cached
    True
    >>> get_defaults(f1)
    (0, [], (1, 2, 3))
    >>> a, b = get_defaults(f1)[1:]
    >>> c, d = f1(0)
    >>> a is c
    True
    >>> b is d
    True
    """
    ret = []
    for i in {}, []:
        def foo(a, b=0, c=i, d=(1,2,3)):
            return c, d
        ret.append(foo)
    return ret

_counter = 0
def counter():
    global _counter
    _counter += 1
    return _counter

def test_defaults_nonliteral_func_call(f):
    """
    >>> f = test_defaults_nonliteral_func_call(counter)
    >>> f()
    1
    >>> get_defaults(f)
    (1,)
    >>> f = test_defaults_nonliteral_func_call(lambda: list())
    >>> f()
    []
    >>> get_defaults(f)
    ([],)
    >>> get_defaults(f)[0] is f()
    True
    """
    def func(a=f()):
        return a
    return func

def assign_defaults_and_check_warnings(func, value=None, delete=False):
    import warnings
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        if delete:
            del func.__defaults__
        else:
            func.__defaults__ = value
        assert len(w) == 1, len(w)
        assert issubclass(w[0].category, RuntimeWarning), w[0].category
        assert "changes to cyfunction.__defaults__" in str(w[0].message), str(w[0].message)

def test_assign_defaults():
    """
    >>> f = test_assign_defaults()
    >>> f.__defaults__
    (5, 10)
    >>> assign_defaults_and_check_warnings(f, value=())
    >>> f.__defaults__
    ()
    >>> assign_defaults_and_check_warnings(f, delete=True)
    >>> f.__defaults__
    >>> f.__defaults__ = "Not a tuple"
    Traceback (most recent call last):
    TypeError: __defaults__ must be set to a tuple object
    """
    def func(a=5, b=10):
        return a, b
    return func


def cy_kwonly_default_args(a, x=1, *, b=2):
    l = m = 1

def assign_kwdefaults_and_check_warnings(func, value):
    import warnings
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        func.__kwdefaults__ = value
        assert len(w) == 1, len(w)
        assert issubclass(w[0].category, RuntimeWarning), w[0].category
        assert "changes to cyfunction.__kwdefaults__" in str(w[0].message), str(w[0].message)

def test_kwdefaults(value):
    """
    >>> cy_kwonly_default_args.__defaults__
    (1,)
    >>> cy_kwonly_default_args.func_defaults
    (1,)

    >>> cy_kwonly_default_args.__kwdefaults__
    {'b': 2}

    >>> test_kwdefaults.__defaults__
    >>> test_kwdefaults.__kwdefaults__

    >>> f = test_kwdefaults(5)
    >>> f.__defaults__
    (1,)
    >>> f.__kwdefaults__
    {'b': 5}
    >>> f.__kwdefaults__ = ()
    Traceback (most recent call last):
    TypeError: __kwdefaults__ must be set to a dict object
    >>> assign_kwdefaults_and_check_warnings(f, None)
    >>> f.__kwdefaults__
    >>> assign_kwdefaults_and_check_warnings(f, {})
    >>> f.__kwdefaults__
    {}
    >>> assign_kwdefaults_and_check_warnings(f, {'a': 2})
    >>> f.__kwdefaults__
    {'a': 2}
    """
    def kwonly_default_args(a, x=1, *, b=value):
        return a, x, b
    return kwonly_default_args


_counter2 = 1.0
def counter2():
    global _counter2
    _counter2 += 1.0
    return _counter2

def test_defaults_fused(cython.floating arg1, cython.floating arg2 = counter2()):
    """
    >>> test_defaults_fused(1.0)
    1.0 2.0
    >>> test_defaults_fused(1.0, 3.0)
    1.0 3.0
    >>> _counter2
    2.0

    >>> get_defaults(test_defaults_fused)
    (2.0,)
    >>> get_defaults(test_defaults_fused[float])
    (2.0,)
    """
    print arg1, arg2

funcs = []
for i in range(10):
    def defaults_fused(cython.floating a, cython.floating b = i):
        return a, b
    funcs.append(defaults_fused)

def test_dynamic_defaults_fused():
    """
    >>> test_dynamic_defaults_fused()
    i 0 func result (1.0, 0.0) defaults (0,)
    i 1 func result (1.0, 1.0) defaults (1,)
    i 2 func result (1.0, 2.0) defaults (2,)
    i 3 func result (1.0, 3.0) defaults (3,)
    i 4 func result (1.0, 4.0) defaults (4,)
    i 5 func result (1.0, 5.0) defaults (5,)
    i 6 func result (1.0, 6.0) defaults (6,)
    i 7 func result (1.0, 7.0) defaults (7,)
    i 8 func result (1.0, 8.0) defaults (8,)
    i 9 func result (1.0, 9.0) defaults (9,)
    """
    for i, f in enumerate(funcs):
        print "i", i, "func result", f(1.0), "defaults", get_defaults(f)


def test_memoryview_none(const unsigned char[:] b=None):
    """
    >>> test_memoryview_none()
    >>> test_memoryview_none(None)
    >>> test_memoryview_none(b'abc')
    97
    """
    if b is None:
        return None
    return b[0]


def test_memoryview_bytes(const unsigned char[:] b=b'xyz'):
    """
    >>> test_memoryview_bytes()
    120
    >>> test_memoryview_bytes(None)
    >>> test_memoryview_bytes(b'abc')
    97
    """
    if b is None:
        return None
    return b[0]


@cython.test_fail_if_path_exists(
    '//NameNode[@entry.in_closure = True]',
    '//NameNode[@entry.from_closure = True]')
def test_func_default_inlined():
    """
    Make sure we don't accidentally generate a closure.

    >>> func = test_func_default_inlined()
    >>> func()
    1
    >>> func(2)
    2
    """
    def default():
        return 1
    def func(arg=default()):
        return arg
    return func


@cython.test_fail_if_path_exists(
    '//NameNode[@entry.in_closure = True]',
    '//NameNode[@entry.from_closure = True]')
def test_func_default_scope():
    """
    Test that the default value expression is evaluated in the outer scope.

    >>> func = test_func_default_scope()
    3
    >>> func()
    [0, 1, 2, 3]
    >>> func(2)
    2
    """
    i = -1
    def func(arg=[ i for i in range(4) ]):
        return arg
    print i  # list comps leak in Py2 mode => i == 3
    return func


def test_func_default_scope_local():
    """
    >>> func = test_func_default_scope_local()
    -1
    >>> func()
    [0, 1, 2, 3]
    >>> func(2)
    2
    """
    i = -1
    def func(arg=list(i for i in range(4))):
        return arg
    print i  # genexprs don't leak
    return func

cdef class C:
    def f1(self, a, b=1, c=[]):
        pass
    def f2(self, a, b=1,/, c=[1]):
        pass
    def f3(self, a, /, b=1, *, c=[1]):
        pass
    cpdef f4(self, a, char*c=NULL):
        pass
    cpdef f5(self, a, str s = "123"):
        pass
    cpdef f6(self, a, int s = 4):
        pass
    cpdef f7(self, a, dict s = {'a':22}):
        pass
    cpdef f8(self, a, list s = [15]):
        pass
    cpdef f9(self, a, int[:] s = None):
        pass
    def f10(self, a, /, b=1, *, int[:] c=None):
        pass


def check_defaults_on_methods_for_introspection():
    """
    >>> C.f1.__defaults__
    (1, [])
    >>> C.f1.__kwdefaults__
    >>> C.f2.__defaults__
    (1, [1])
    >>> C.f2.__kwdefaults__
    >>> C.f3.__defaults__
    (1,)
    >>> C.f3.__kwdefaults__
    {'c': [1]}
    >>> C.f4.__defaults__
    >>> C.f4.__kwdefaults__
    >>> C.f5.__defaults__
    ('123',)
    >>> C.f5.__kwdefaults__
    >>> C.f6.__defaults__
    (4,)
    >>> C.f6.__kwdefaults__
    >>> C.f7.__defaults__
    ({'a': 22},)
    >>> C.f7.__kwdefaults__
    >>> C.f8.__defaults__
    ([15],)
    >>> C.f8.__kwdefaults__
    >>> C.f9.__defaults__
    (None,)
    >>> C.f9.__kwdefaults__
    >>> C.f10.__defaults__
    (1,)
    >>> C.f10.__kwdefaults__
    {'c': None}
    """
    pass

cdef extern from *:
    """
    #define TWO 2.0
    #define THREE 3.0
    #define FOUR 4.0
    #define FIVE 5.0
    #define SIX 6.0
    #define SEVEN 7.0
    #define EIGHT 8.0
    #define NINE 9.0
    """
    double TWO, THREE, FOUR, FIVE, SIX, SEVEN, EIGHT, NINE

# https://github.com/cython/cython/issues/6843
# This is to do with lexical sorting of fields called arg0, arg1, arg10, arg2, ...
def test_eleven_defaults(a=[], b=set(), c={}, double d=TWO, double e=THREE,
                      double f=FOUR, double g=FIVE, double h=SIX, double i=SEVEN, double j=EIGHT,
                      double k=NINE):
    """
    >>> test_eleven_defaults()
    [] set() {} 2.0 3.0 4.0 5.0 6.0 7.0 8.0
    """
    print a, b, c, d, e, f, g, h, i, j



# mode: compile
# tag: pep492

# make sure async iterators also compile correctly without using 'await'

cdef class AsyncIter:
    cdef long i
    cdef long aiter_calls
    cdef long max_iter_calls

    def __init__(self, long max_iter_calls=1):
        self.i = 0
        self.aiter_calls = 0
        self.max_iter_calls = max_iter_calls

    def __aiter__(self):
        self.aiter_calls += 1
        return self

    async def __anext__(self):
        self.i += 1
        assert self.aiter_calls <= self.max_iter_calls

        if self.i > 10:
            raise StopAsyncIteration

        return self.i, self.i



def iterate_string():
    cdef unicode ustring = u'Hello world'

    # NOTE: no typing required for 'uchar' !
    for uchar in ustring:
        if uchar == u'A':
            print("Found the letter A")



# mode: error

# invalid syntax (not handled by the parser)

def syntax1():
    a = b = c = d = e = f = g = h = i = 1 # prevent undefined names

    *a

    *1

    *"abc"

    *a*b

    [*a, *b]

    (a, b, *c, d, e, f, *g, h, i)
    [a, b, *c, d, e, f, *g, h, i]
    {a, b, *c, d, e, f, *g, h, i}


def syntax2():
    list_of_sequences = [[1,2], [3,4]]

    for *a,*b in list_of_sequences:
        pass


def types(l):
    cdef int a,b
    a, *b = (1,2,3,4)
    a, *b = l


_ERRORS = u"""
# syntax1()
 8: 4: starred expression is not allowed here
10: 4: starred expression is not allowed here
12: 4: starred expression is not allowed here
14: 4: starred expression is not allowed here

# syntax2()
26:11: more than 1 starred expression in assignment

# types()
32:15: Cannot coerce list to type 'int'
33:8: starred target must have Python object (list) type
"""



# mode: compile
# cython: test_assert_c_code_has = CythonPyMutexPublicCheck

# Because this file uses PyMutex in a pxd file, it must generate the CythonPyMutexPublicCheck sanity
# check for the Stable ABI

cdef class C:
    pass  # see pxd file



# mode: run
# tag: forin

import sys
import cython

try:
    from builtins import next
except ImportError:
    def next(it):
        return it.next()

def for_in_pyiter_pass(it):
    """
    >>> it = Iterable(5)
    >>> for_in_pyiter_pass(it)
    >>> next(it)
    Traceback (most recent call last):
    StopIteration
    """
    for item in it:
        pass

def for_in_pyiter(it):
    """
    >>> for_in_pyiter(Iterable(5))
    [0, 1, 2, 3, 4]
    """
    l = []
    for item in it:
        l.append(item)
    return l

def for_in_list():
    """
    >>> for_in_pyiter([1,2,3,4,5])
    [1, 2, 3, 4, 5]
    """

@cython.test_assert_path_exists('//TupleNode//IntNode')
@cython.test_fail_if_path_exists('//ListNode//IntNode')
def for_in_literal_list():
    """
    >>> for_in_literal_list()
    [1, 2, 3, 4]
    """
    l = []
    for i in [1,2,3,4]:
        l.append(i)
    return l

@cython.test_assert_path_exists('//TupleNode//IntNode')
@cython.test_fail_if_path_exists('//ListNode//IntNode')
def for_in_literal_mult_list():
    """
    >>> for_in_literal_mult_list()
    [1, 2, 3, 4, 1, 2, 3, 4, 1, 2, 3, 4]
    """
    l = []
    for i in [1,2,3,4] * 3:
        l.append(i)
    return l


def listcomp_over_multiplied_constant_tuple():
    """
    >>> listcomp_over_multiplied_constant_tuple()
    [[], [1, 2, 3], [1, 2, 3, 1, 2, 3], [1, 2, 3, 1, 2, 3, 1, 2, 3], [1, 2, 3, 1, 2, 3]]
    """
    return [
        [i for i in (1, 2, 3) * 0],
        [i for i in (1, 2, 3) * 1],
        [i for i in (1, 2, 3) * 2],
        [i for i in (1, 2, 3) * 3],
        [i for i in (1, 2, 3) * 2],
    ]


@cython.test_assert_path_exists('//ReturnStatNode//ForInStatNode//TupleNode')
@cython.test_fail_if_path_exists('//ReturnStatNode//ForInStatNode//ListNode')
def listcomp_over_multiplied_constant_list():
    """
    >>> listcomp_over_multiplied_constant_list()
    [[], [1, 2, 3], [1, 2, 3, 1, 2, 3], [1, 2, 3, 1, 2, 3, 1, 2, 3], [1, 2, 3, 1, 2, 3]]
    """
    return [
        [i for i in [1, 2, 3] * 0],
        [i for i in [1, 2, 3] * 1],
        [i for i in [1, 2, 3] * 2],
        [i for i in [1, 2, 3] * 3],
        [i for i in [1, 2, 3] * 2],
    ]


class Iterable(object):
    """
    >>> for_in_pyiter(Iterable(5))
    [0, 1, 2, 3, 4]
    """
    def __init__(self, N):
        self.N = N
        self.i = 0
    def __iter__(self):
        return self
    def __next__(self):
        if self.i < self.N:
            i = self.i
            self.i += 1
            return i
        raise StopIteration
    next = __next__


class NextReplacingIterable(object):
    def __init__(self):
        self.i = 0
    def __iter__(self):
        return self

    def __next__(self):
        if self.i > 5:
            raise StopIteration
        self.i += 1
        self.__next__ = self.next2
        return 1
    def next2(self):
        self.__next__ = self.next3
        return 2
    def next3(self):
        del self.__next__
        raise StopIteration


def for_in_next_replacing_iter():
    """
    >>> for_in_pyiter(NextReplacingIterable())
    [1, 1, 1, 1, 1, 1]
    """


def for_in_gen(N):
    """
    >>> for_in_pyiter(for_in_gen(10))
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    """
    for i in range(N):
        yield i

def for_in_range_invalid_arg_count():
    """
    >>> for_in_range_invalid_arg_count()     # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...
    """
    for i in range(1, 2, 3, 4):
        pass



# mode: run
# tag: exceptions, tryfinally

import sys


def test_finally_c():
    """
    >>> def test_finally_py():
    ...     try:
    ...         raise AttributeError()
    ...     finally:
    ...         raise KeyError()

    >>> try:
    ...     test_finally_py()
    ... except KeyError:
    ...     print(sys.exc_info()[0] is KeyError or sys.exc_info()[0])
    ...     print(isinstance(sys.exc_info()[1].__context__, AttributeError)
    ...           or sys.exc_info()[1].__context__)
    True
    True

    >>> try:
    ...     test_finally_c()
    ... except KeyError:
    ...     print(sys.exc_info()[0] is KeyError or sys.exc_info()[0])
    ...     print(isinstance(sys.exc_info()[1].__context__, AttributeError)
    ...           or sys.exc_info()[1].__context__)
    True
    True
    """
    try:
        raise AttributeError()
    finally:
        raise KeyError()



# cython: language_level=3str
# mode: run

cimport cython

from operator import lshift as py_lshift, rshift as py_rshift


def bigint(x):
    # avoid 'L' postfix in Py2.x
    print(str(x).rstrip('L'))

def bigints(x):
    # avoid 'L' postfix in Py2.x
    print(str(x).replace('L', ''))


@cython.test_assert_path_exists('//BitwiseOrNode')
def or_obj(obj2, obj3):
    """
    >>> or_obj(2, 3)
    3
    """
    obj1 = obj2 | obj3
    return obj1


@cython.test_fail_if_path_exists('//BitwiseOrNode')
def or_int(obj2):
    """
    >>> or_int(0)
    16
    >>> or_int(1)
    17
    >>> or_int(16)
    16
    """
    obj1 = obj2 | 0x10
    return obj1


@cython.test_assert_path_exists('//IntBinopNode')
def xor_obj(obj2, obj3):
    """
    >>> xor_obj(2, 3)
    1
    """
    obj1 = obj2 ^ obj3
    return obj1


@cython.test_fail_if_path_exists('//IntBinopNode')
def xor_int(obj2):
    """
    >>> xor_int(0)
    16
    >>> xor_int(2)
    18
    >>> xor_int(16)
    0
    """
    obj1 = obj2 ^ 0x10
    return obj1


@cython.test_assert_path_exists('//IntBinopNode')
def and_obj(obj2, obj3):
    """
    >>> and_obj(2, 3)
    2
    """
    obj1 = obj2 & obj3
    return obj1


@cython.test_fail_if_path_exists('//IntBinopNode')
def and_int(obj2):
    """
    >>> and_int(0)
    0
    >>> and_int(1)
    0
    >>> and_int(18)
    16
    >>> and_int(-1)
    16
    """
    obj1 = obj2 & 0x10
    return obj1


@cython.test_fail_if_path_exists('//IntBinopNode')
def and_int2(obj2):
    # On Python 3.10 and earlier, from_bytes produces a non-canonical
    # 0 that caused trouble when &ing with a constant.
    """
    >>> and_int2(1337)
    57
    >>> and_int2(int.from_bytes(b'\\x00', 'big'))
    0
    """
    obj1 = obj2 & 0xff
    return obj1


@cython.test_assert_path_exists('//IntBinopNode')
def lshift_obj(obj2, obj3):
    """
    >>> lshift_obj(2, 3)
    16
    """
    obj1 = obj2 << obj3
    return obj1


@cython.test_assert_path_exists('//IntBinopNode')
def rshift_obj(obj2, obj3):
    """
    >>> rshift_obj(2, 3)
    0
    """
    obj1 = obj2 >> obj3
    return obj1


@cython.test_assert_path_exists('//IntBinopNode')
def rshift_int_obj(obj3):
    """
    >>> rshift_int_obj(3)
    0
    >>> rshift_int_obj(2)
    0
    >>> rshift_int_obj(1)
    1
    >>> rshift_int_obj(0)
    2
    >>> rshift_int_obj(-1)
    Traceback (most recent call last):
    ValueError: negative shift count
    """
    obj1 = 2 >> obj3
    return obj1


@cython.test_fail_if_path_exists('//IntBinopNode')
def rshift_int(obj2):
    """
    >>> rshift_int(0)
    0
    >>> rshift_int(2)
    0

    >>> rshift_int(27)
    3
    >>> (-27) >> 3
    -4
    >>> rshift_int(-27)
    -4

    >>> rshift_int(32)
    4
    >>> (-32) >> 3
    -4
    >>> rshift_int(-32)
    -4

    >>> (2**28) >> 3
    33554432
    >>> rshift_int(2**28)
    33554432
    >>> (-2**28) >> 3
    -33554432
    >>> rshift_int(-2**28)
    -33554432

    >>> (2**30) >> 3
    134217728
    >>> rshift_int(2**30)
    134217728
    >>> rshift_int(-2**30)
    -134217728

    >>> bigint((2**60) >> 3)
    144115188075855872
    >>> bigint(rshift_int(2**60))
    144115188075855872
    >>> bigint(rshift_int(-2**60))
    -144115188075855872

    >>> rshift_int(-1)
    -1
    >>> (-1) >> 3
    -1
    """
    obj1 = obj2 >> 3
    return obj1


@cython.test_assert_path_exists(
    '//SingleAssignmentNode//IntBinopNode',
    '//SingleAssignmentNode//PythonCapiCallNode',
)
def lshift_int(obj):
    """
    >>> lshift_int(0)
    (0, 0, 0, 0)
    >>> bigints(lshift_int(1))
    (8, 2147483648, 9223372036854775808, 10633823966279326983230456482242756608)
    >>> bigints(lshift_int(-1))
    (-8, -2147483648, -9223372036854775808, -10633823966279326983230456482242756608)
    >>> bigints(lshift_int(2))
    (16, 4294967296, 18446744073709551616, 21267647932558653966460912964485513216)

    >>> bigints(lshift_int(27))
    (216, 57982058496, 249031044995078946816, 287113247089541828547222325020554428416)
    >>> (-27) << 3
    -216
    >>> bigints(lshift_int(-27))
    (-216, -57982058496, -249031044995078946816, -287113247089541828547222325020554428416)

    >>> bigints(lshift_int(32))
    (256, 68719476736, 295147905179352825856, 340282366920938463463374607431768211456)
    >>> (-32) << 3
    -256
    >>> bigints(lshift_int(-32))
    (-256, -68719476736, -295147905179352825856, -340282366920938463463374607431768211456)

    >>> bigint((2**28) << 3)
    2147483648
    >>> bigints(lshift_int(2**28))
    (2147483648, 576460752303423488, 2475880078570760549798248448, 2854495385411919762116571938898990272765493248)
    >>> bigint((-2**28) << 3)
    -2147483648
    >>> bigints(lshift_int(-2**28))
    (-2147483648, -576460752303423488, -2475880078570760549798248448, -2854495385411919762116571938898990272765493248)

    >>> bigint((2**30) << 3)
    8589934592
    >>> bigints(lshift_int(2**30))
    (8589934592, 2305843009213693952, 9903520314283042199192993792, 11417981541647679048466287755595961091061972992)
    >>> bigints(lshift_int(-2**30))
    (-8589934592, -2305843009213693952, -9903520314283042199192993792, -11417981541647679048466287755595961091061972992)

    >>> bigint((2**60) << 3)
    9223372036854775808
    >>> bigints(lshift_int(2**60))
    (9223372036854775808, 2475880078570760549798248448, 10633823966279326983230456482242756608, 12259964326927110866866776217202473468949912977468817408)
    >>> bigints(lshift_int(-2**60))
    (-9223372036854775808, -2475880078570760549798248448, -10633823966279326983230456482242756608, -12259964326927110866866776217202473468949912977468817408)
    """
    r1 = obj << 3
    r2 = obj << 31
    r3 = obj << 63
    r4 = obj << 123
    return r1, r2, r3, r4


@cython.test_assert_path_exists(
    '//IntBinopNode',
    '//BitwiseOrNode//IntBinopNode',
)
def mixed_obj(obj2, obj3):
    """
    >>> mixed_obj(2, 3)
    16
    """
    obj1 = obj2 << obj3 | obj2 >> obj3
    return obj1


@cython.test_assert_path_exists(
    '//BitwiseOrNode',
    '//BitwiseOrNode//PythonCapiCallNode',
)
@cython.test_fail_if_path_exists(
    '//IntBinopNode//IntBinopNode',
)
def mixed_int(obj2):
    """
    >>> mixed_int(0)
    16
    >>> mixed_int(2)
    18
    >>> mixed_int(16)
    0
    >>> mixed_int(17)
    1
    """
    obj1 = (obj2 ^ 0x10) | (obj2 & 0x01)
    return obj1


@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_fail_if_path_exists(
    '//IntBinopNode',
    '//PrimaryCmpNode',
)
def equals(obj2):
    """
    >>> equals(2)
    True
    >>> equals(0)
    False
    >>> equals(-1)
    False
    """
    result = obj2 == 2
    return result


@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_fail_if_path_exists(
    '//IntBinopNode',
    '//PrimaryCmpNode',
)
def not_equals(obj2):
    """
    >>> not_equals(2)
    False
    >>> not_equals(0)
    True
    >>> not_equals(-1)
    True
    """
    result = obj2 != 2
    return result


@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_assert_path_exists('//PrimaryCmpNode')
def equals_many(obj2):
    """
    >>> equals_many(-2)
    (False, False, False, False, False, False, False, False, False, False, False, False, False, False, False)
    >>> equals_many(0)
    (True, False, False, False, False, False, False, False, False, False, False, False, False, False, False)
    >>> equals_many(1)
    (False, True, False, False, False, False, False, False, False, False, False, False, False, False, False)
    >>> equals_many(-1)
    (False, False, True, False, False, False, False, False, False, False, False, False, False, False, False)
    >>> equals_many(2**30)
    (False, False, False, True, False, False, False, False, False, False, False, False, False, False, False)
    >>> equals_many(-2**30)
    (False, False, False, False, True, False, False, False, False, False, False, False, False, False, False)
    >>> equals_many(2**30-1)
    (False, False, False, False, False, True, False, False, False, False, False, False, False, False, False)
    >>> equals_many(-2**30+1)
    (False, False, False, False, False, False, True, False, False, False, False, False, False, False, False)
    >>> equals_many(2**32)
    (False, False, False, False, False, False, False, True, False, False, False, False, False, False, False)
    >>> equals_many(-2**32)
    (False, False, False, False, False, False, False, False, True, False, False, False, False, False, False)
    >>> equals_many(2**45-1)
    (False, False, False, False, False, False, False, False, False, True, False, False, False, False, False)
    >>> equals_many(-2**45+1)
    (False, False, False, False, False, False, False, False, False, False, True, False, False, False, False)
    >>> equals_many(2**64)
    (False, False, False, False, False, False, False, False, False, False, False, True, False, False, False)
    >>> equals_many(-2**64)
    (False, False, False, False, False, False, False, False, False, False, False, False, True, False, False)
    >>> equals_many(2**64-1)
    (False, False, False, False, False, False, False, False, False, False, False, False, False, True, False)
    >>> equals_many(-2**64+1)
    (False, False, False, False, False, False, False, False, False, False, False, False, False, False, True)
    """
    cdef bint x, a, b, c, d, e, f, g, h, i, j, k, l, m, n, o
    a = obj2 == 0
    x = 0 == obj2
    assert a == x
    b = obj2 == 1
    x = 1 == obj2
    assert b == x
    c = obj2 == -1
    x = -1 == obj2
    assert c == x
    d = obj2 == 2**30
    x = 2**30 == obj2
    assert d == x
    e = obj2 == -2**30
    x = -2**30 == obj2
    assert e == x
    f = obj2 == 2**30-1
    x = 2**30-1 == obj2
    assert f == x
    g = obj2 == -2**30+1
    x = -2**30+1 == obj2
    assert g == x
    h = obj2 == 2**32
    x = 2**32 == obj2
    assert h == x
    i = obj2 == -2**32
    x = -2**32 == obj2
    assert i == x
    j = obj2 == 2**45-1
    x = 2**45-1 == obj2
    assert j == x
    k = obj2 == -2**45+1
    x = -2**45+1 == obj2
    assert k == x
    l = obj2 == 2**64
    x = 2**64 == obj2
    assert l == x
    m = obj2 == -2**64
    x = -2**64 == obj2
    assert m == x
    n = obj2 == 2**64-1
    x = 2**64-1 == obj2
    assert n == x
    o = obj2 == -2**64+1
    x = -2**64+1 == obj2
    assert o == x
    return (a, b, c, d, e, f, g, h, i, j, k, l, m, n, o)


@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_assert_path_exists('//PrimaryCmpNode')
def not_equals_many(obj2):
    """
    >>> not_equals_many(-2)
    (False, False, False, False, False, False, False, False, False, False, False, False, False, False, False)
    >>> not_equals_many(0)
    (True, False, False, False, False, False, False, False, False, False, False, False, False, False, False)
    >>> not_equals_many(1)
    (False, True, False, False, False, False, False, False, False, False, False, False, False, False, False)
    >>> not_equals_many(-1)
    (False, False, True, False, False, False, False, False, False, False, False, False, False, False, False)
    >>> not_equals_many(2**30)
    (False, False, False, True, False, False, False, False, False, False, False, False, False, False, False)
    >>> not_equals_many(-2**30)
    (False, False, False, False, True, False, False, False, False, False, False, False, False, False, False)
    >>> not_equals_many(2**30-1)
    (False, False, False, False, False, True, False, False, False, False, False, False, False, False, False)
    >>> not_equals_many(-2**30+1)
    (False, False, False, False, False, False, True, False, False, False, False, False, False, False, False)
    >>> not_equals_many(2**32)
    (False, False, False, False, False, False, False, True, False, False, False, False, False, False, False)
    >>> not_equals_many(-2**32)
    (False, False, False, False, False, False, False, False, True, False, False, False, False, False, False)
    >>> not_equals_many(2**45-1)
    (False, False, False, False, False, False, False, False, False, True, False, False, False, False, False)
    >>> not_equals_many(-2**45+1)
    (False, False, False, False, False, False, False, False, False, False, True, False, False, False, False)
    >>> not_equals_many(2**64)
    (False, False, False, False, False, False, False, False, False, False, False, True, False, False, False)
    >>> not_equals_many(-2**64)
    (False, False, False, False, False, False, False, False, False, False, False, False, True, False, False)
    >>> not_equals_many(2**64-1)
    (False, False, False, False, False, False, False, False, False, False, False, False, False, True, False)
    >>> not_equals_many(-2**64+1)
    (False, False, False, False, False, False, False, False, False, False, False, False, False, False, True)
    """
    cdef bint a, b, c, d, e, f, g, h, i, j, k, l, m, n, o
    a = obj2 != 0
    x = 0 != obj2
    assert a == x
    b = obj2 != 1
    x = 1 != obj2
    assert b == x
    c = obj2 != -1
    x = -1 != obj2
    assert c == x
    d = obj2 != 2**30
    x = 2**30 != obj2
    assert d == x
    e = obj2 != -2**30
    x = -2**30 != obj2
    assert e == x
    f = obj2 != 2**30-1
    x = 2**30-1 != obj2
    assert f == x
    g = obj2 != -2**30+1
    x = -2**30+1 != obj2
    assert g == x
    h = obj2 != 2**32
    x = 2**32 != obj2
    assert h == x
    i = obj2 != -2**32
    x = -2**32 != obj2
    assert i == x
    j = obj2 != 2**45-1
    x = 2**45-1 != obj2
    assert j == x
    k = obj2 != -2**45+1
    x = -2**45+1 != obj2
    assert k == x
    l = obj2 != 2**64
    x = 2**64 != obj2
    assert l == x
    m = obj2 != -2**64
    x = -2**64 != obj2
    assert m == x
    n = obj2 != 2**64-1
    x = 2**64-1 != obj2
    assert n == x
    o = obj2 != -2**64+1
    x = -2**64+1 != obj2
    assert o == x
    return tuple(not x for x in (a, b, c, d, e, f, g, h, i, j, k, l, m, n, o))


@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_fail_if_path_exists(
    '//IntBinopNode',
    '//PrimaryCmpNode',
)
def equals_zero(obj2):
    """
    >>> equals_zero(2)
    False
    >>> equals_zero(0)
    True
    >>> equals_zero(-1)
    False
    """
    result = obj2 == 0
    return result


def truthy(obj2):
    """
    >>> truthy(2)
    True
    >>> truthy(0)
    False
    >>> truthy(-1)
    True
    """
    if obj2:
        return True
    else:
        return False

@cython.test_fail_if_path_exists("//CoerceToBooleanNode")
@cython.test_fail_if_path_exists("//CoerceToPyTypeNode")
def test_avoid_if_coercion(obj):
    if obj == 1:  # this should not go through a Python intermediate
        return True
    else:
        return False

@cython.test_fail_if_path_exists('//AddNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_add_int(obj2: int):
    """
    >>> pure_add_int(1)
    (2, 2)
    """
    res1 = obj2 + 1
    res2 = 1 + obj2
    return res1, res2

@cython.test_fail_if_path_exists('//SubNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_sub_int(obj2: int):
    """
    >>> pure_sub_int(1)
    (0, 0)
    """
    res1 = obj2 - 1
    res2 = 1 - obj2
    return res1, res2

@cython.test_fail_if_path_exists('//MulNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_mul_int(obj2: int):
    """
    >>> pure_mul_int(2)
    (4, 4)
    """
    res1 = obj2 * 2
    res2 = 2 * obj2
    return res1, res2

@cython.test_fail_if_path_exists('//PrimaryCmpNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_eq_int(obj2: int):
    """
    >>> pure_eq_int(2)
    (True, True)
    >>> pure_eq_int(3)
    (False, False)
    """
    res1 = obj2 == 2
    res2 = 2 == obj2
    return res1, res2

@cython.test_fail_if_path_exists('//PrimaryCmpNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_ne_int(obj2: int):
    """
    >>> pure_ne_int(2)
    (False, False)
    >>> pure_ne_int(3)
    (True, True)
    """
    res1 = obj2 != 2
    res2 = 2 != obj2
    return res1, res2

@cython.test_fail_if_path_exists('//IntBinopNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_and_int(obj2: int):
    """
    >>> pure_and_int(1)
    (0, 0)
    >>> pure_and_int(3)
    (2, 2)
    """
    res1 = obj2 & 2
    res2 = 2 & obj2
    return res1, res2

@cython.test_fail_if_path_exists('//IntBinopNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_or_int(obj2: int):
    """
    >>> pure_or_int(1)
    (3, 3)
    >>> pure_or_int(0)
    (2, 2)
    """
    res1 = obj2 | 2
    res2 = 2 | obj2
    return res1, res2

@cython.test_fail_if_path_exists('//IntBinopNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_xor_int(obj2: int):
    """
    >>> pure_xor_int(1)
    (3, 3)
    >>> pure_xor_int(3)
    (1, 1)
    """
    res1 = obj2 ^ 2
    res2 = 2 ^ obj2
    return res1, res2

@cython.test_fail_if_path_exists('//IntBinopNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_rshift_int(obj2: int):
    """
    >>> pure_rshift_int(8)
    4
    """
    res = obj2 >> 1
    return res

@cython.test_fail_if_path_exists('//IntBinopNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_lshift_int(obj2: int):
    """
    >>> pure_lshift_int(8)
    16
    """
    res = obj2 << 1
    return res

@cython.test_fail_if_path_exists('//IntBinopNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_mod_int(obj2: int):
    """
    >>> pure_mod_int(3)
    1
    """
    res = obj2 % 2
    return res

@cython.test_fail_if_path_exists('//DivNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_floordiv_int(obj2: int):
    """
    >>> pure_floordiv_int(3)
    1
    """
    res = obj2 // 2
    return res

import sys


@cython.test_fail_if_path_exists('//DivNode')
@cython.test_fail_if_path_exists('//NumBinopNode')
@cython.test_fail_if_path_exists('//BinopNode')
@cython.test_assert_path_exists('//PythonCapiFunctionNode')
def pure_truediv_int(obj2: int):
    """
    >>> pure_truediv_int(5)
    2.5
    """
    res = obj2 / 2
    return res


def negative_rhs_lshift(obj: int):
    """
    >>> negative_rhs_lshift(5)
    Traceback (most recent call last):
        ...
    ValueError: negative shift count
    """
    return (obj << (-1))

def negative_rhs_rshift(obj: int):
    """
    >>> negative_rhs_rshift(5)
    Traceback (most recent call last):
        ...
    ValueError: negative shift count
    """
    return (obj >> (-1))


def big_lshift(obj):
    """
    >>> big_lshift(1)
    1180591620717411303424
    >>> big_lshift(-1)
    -1180591620717411303424
    """
    return obj << 70

def somewhat_big_lshift(obj):
    """
    Note that on MSVC this is bigger than a long

    >>> somewhat_big_lshift(1)
    (1099511627776, 4294967296)
    >>> somewhat_big_lshift(-1)
    (-1099511627776, -4294967296)
    """
    return obj << 40, obj << 32

def big_rshift(obj):
    """
    >>> big_rshift(1)
    0
    >>> big_rshift(-1)
    -1
    >>> big_rshift(2**80)
    1024
    >>> big_rshift(-(2**80))
    -1024
    """
    return obj >> 70

def somewhat_big_rshift(obj):
    """
    Note that on MSVC this is bigger than a long

    >>> somewhat_big_rshift(1)
    (0, 0)
    >>> somewhat_big_rshift(-1)
    (-1, -1)
    >>> somewhat_big_rshift(0x6f)
    (0, 0)
    >>> somewhat_big_rshift(-0x6f)
    (-1, -1)
    >>> somewhat_big_rshift(2**50)
    (1024, 262144)
    >>> somewhat_big_rshift(-(2**50))
    (-1024, -262144)
    """
    return obj >> 40, obj >> 32

cdef compare_lshift(obj, shift, cython_result):
    py_result = py_lshift(obj, shift)
    assert cython_result == py_result, f"{obj} << {shift} == {cython_result} != {py_result}"

cdef compare_rshift(obj, shift, cython_result):
    py_result = py_rshift(obj, shift)
    assert cython_result == py_result, f"{obj} >> {shift} == {cython_result} != {py_result}"

cdef compare_both_shifts(obj, shift, cython_lresult, cython_rresult):
    compare_lshift(obj, shift, cython_lresult)
    compare_rshift(obj, shift, cython_rresult)

def integer_shifts(obj):
    """
    >>> integer_shifts(0)
    >>> integer_shifts(1)
    >>> integer_shifts(-1)
    >>> integer_shifts(0xff)
    >>> integer_shifts(-0xff)
    >>> integer_shifts(0xff00)
    >>> integer_shifts(-0xff00)
    >>> integer_shifts(0xff0000)
    >>> integer_shifts(-0xff0000)
    >>> integer_shifts(0xff000000)
    >>> integer_shifts(-0xff000000)
    """
    compare_both_shifts(obj, 0, obj << 0, obj >> 0)
    compare_both_shifts(obj, 1, obj << 1, obj >> 1)
    compare_both_shifts(obj, 2, obj << 2, obj >> 2)
    compare_both_shifts(obj, 3, obj << 3, obj >> 3)
    compare_both_shifts(obj, 4, obj << 4, obj >> 4)
    compare_both_shifts(obj, 5, obj << 5, obj >> 5)
    compare_both_shifts(obj, 6, obj << 6, obj >> 6)
    compare_both_shifts(obj, 7, obj << 7, obj >> 7)
    compare_both_shifts(obj, 8, obj << 8, obj >> 8)
    compare_both_shifts(obj, 9, obj << 9, obj >> 9)
    compare_both_shifts(obj, 10, obj << 10, obj >> 10)
    compare_both_shifts(obj, 11, obj << 11, obj >> 11)
    compare_both_shifts(obj, 12, obj << 12, obj >> 12)
    compare_both_shifts(obj, 13, obj << 13, obj >> 13)
    compare_both_shifts(obj, 14, obj << 14, obj >> 14)
    compare_both_shifts(obj, 15, obj << 15, obj >> 15)
    compare_both_shifts(obj, 16, obj << 16, obj >> 16)
    compare_both_shifts(obj, 17, obj << 17, obj >> 17)
    compare_both_shifts(obj, 18, obj << 18, obj >> 18)
    compare_both_shifts(obj, 19, obj << 19, obj >> 19)
    compare_both_shifts(obj, 20, obj << 20, obj >> 20)
    compare_both_shifts(obj, 21, obj << 21, obj >> 21)
    compare_both_shifts(obj, 22, obj << 22, obj >> 22)
    compare_both_shifts(obj, 23, obj << 23, obj >> 23)
    compare_both_shifts(obj, 24, obj << 24, obj >> 24)
    compare_both_shifts(obj, 25, obj << 25, obj >> 25)
    compare_both_shifts(obj, 26, obj << 26, obj >> 26)
    compare_both_shifts(obj, 27, obj << 27, obj >> 27)
    compare_both_shifts(obj, 28, obj << 28, obj >> 28)
    compare_both_shifts(obj, 29, obj << 29, obj >> 29)
    compare_both_shifts(obj, 30, obj << 30, obj >> 30)
    compare_both_shifts(obj, 31, obj << 31, obj >> 31)
    compare_both_shifts(obj, 32, obj << 32, obj >> 32)
    compare_both_shifts(obj, 33, obj << 33, obj >> 33)
    compare_both_shifts(obj, 34, obj << 34, obj >> 34)
    compare_both_shifts(obj, 35, obj << 35, obj >> 35)
    compare_both_shifts(obj, 36, obj << 36, obj >> 36)
    compare_both_shifts(obj, 37, obj << 37, obj >> 37)
    compare_both_shifts(obj, 38, obj << 38, obj >> 38)
    compare_both_shifts(obj, 39, obj << 39, obj >> 39)
    compare_both_shifts(obj, 40, obj << 40, obj >> 40)
    compare_both_shifts(obj, 41, obj << 41, obj >> 41)
    compare_both_shifts(obj, 42, obj << 42, obj >> 42)
    compare_both_shifts(obj, 43, obj << 43, obj >> 43)
    compare_both_shifts(obj, 44, obj << 44, obj >> 44)
    compare_both_shifts(obj, 45, obj << 45, obj >> 45)
    compare_both_shifts(obj, 46, obj << 46, obj >> 46)
    compare_both_shifts(obj, 47, obj << 47, obj >> 47)
    compare_both_shifts(obj, 48, obj << 48, obj >> 48)
    compare_both_shifts(obj, 49, obj << 49, obj >> 49)
    compare_both_shifts(obj, 50, obj << 50, obj >> 50)
    compare_both_shifts(obj, 51, obj << 51, obj >> 51)
    compare_both_shifts(obj, 52, obj << 52, obj >> 52)
    compare_both_shifts(obj, 53, obj << 53, obj >> 53)
    compare_both_shifts(obj, 54, obj << 54, obj >> 54)
    compare_both_shifts(obj, 55, obj << 55, obj >> 55)
    compare_both_shifts(obj, 56, obj << 56, obj >> 56)
    compare_both_shifts(obj, 57, obj << 57, obj >> 57)
    compare_both_shifts(obj, 58, obj << 58, obj >> 58)
    compare_both_shifts(obj, 59, obj << 59, obj >> 59)
    compare_both_shifts(obj, 60, obj << 60, obj >> 60)
    compare_both_shifts(obj, 61, obj << 61, obj >> 61)
    compare_both_shifts(obj, 62, obj << 62, obj >> 62)
    compare_both_shifts(obj, 63, obj << 63, obj >> 63)
    compare_both_shifts(obj, 64, obj << 64, obj >> 64)
    compare_both_shifts(obj, 65, obj << 65, obj >> 65)
    compare_both_shifts(obj, 66, obj << 66, obj >> 66)
    compare_both_shifts(obj, 67, obj << 67, obj >> 67)
    compare_both_shifts(obj, 68, obj << 68, obj >> 68)
    compare_both_shifts(obj, 69, obj << 69, obj >> 69)
    compare_both_shifts(obj, 70, obj << 70, obj >> 70)
    compare_both_shifts(obj, 71, obj << 71, obj >> 71)
    compare_both_shifts(obj, 72, obj << 72, obj >> 72)
    compare_both_shifts(obj, 73, obj << 73, obj >> 73)
    compare_both_shifts(obj, 74, obj << 74, obj >> 74)
    compare_both_shifts(obj, 75, obj << 75, obj >> 75)
    compare_both_shifts(obj, 76, obj << 76, obj >> 76)
    compare_both_shifts(obj, 77, obj << 77, obj >> 77)
    compare_both_shifts(obj, 78, obj << 78, obj >> 78)
    compare_both_shifts(obj, 79, obj << 79, obj >> 79)
    compare_both_shifts(obj, 80, obj << 80, obj >> 80)
    compare_both_shifts(obj, 81, obj << 81, obj >> 81)
    compare_both_shifts(obj, 82, obj << 82, obj >> 82)
    compare_both_shifts(obj, 83, obj << 83, obj >> 83)
    compare_both_shifts(obj, 84, obj << 84, obj >> 84)
    compare_both_shifts(obj, 85, obj << 85, obj >> 85)
    compare_both_shifts(obj, 86, obj << 86, obj >> 86)
    compare_both_shifts(obj, 87, obj << 87, obj >> 87)
    compare_both_shifts(obj, 88, obj << 88, obj >> 88)
    compare_both_shifts(obj, 89, obj << 89, obj >> 89)
    compare_both_shifts(obj, 90, obj << 90, obj >> 90)
    compare_both_shifts(obj, 91, obj << 91, obj >> 91)
    compare_both_shifts(obj, 92, obj << 92, obj >> 92)
    compare_both_shifts(obj, 93, obj << 93, obj >> 93)
    compare_both_shifts(obj, 94, obj << 94, obj >> 94)
    compare_both_shifts(obj, 95, obj << 95, obj >> 95)
    compare_both_shifts(obj, 96, obj << 96, obj >> 96)
    compare_both_shifts(obj, 97, obj << 97, obj >> 97)
    compare_both_shifts(obj, 98, obj << 98, obj >> 98)
    compare_both_shifts(obj, 99, obj << 99, obj >> 99)
    compare_both_shifts(obj, 100, obj << 100, obj >> 100)
    compare_both_shifts(obj, 101, obj << 101, obj >> 101)
    compare_both_shifts(obj, 102, obj << 102, obj >> 102)
    compare_both_shifts(obj, 103, obj << 103, obj >> 103)
    compare_both_shifts(obj, 104, obj << 104, obj >> 104)
    compare_both_shifts(obj, 105, obj << 105, obj >> 105)
    compare_both_shifts(obj, 106, obj << 106, obj >> 106)
    compare_both_shifts(obj, 107, obj << 107, obj >> 107)
    compare_both_shifts(obj, 108, obj << 108, obj >> 108)
    compare_both_shifts(obj, 109, obj << 109, obj >> 109)
    compare_both_shifts(obj, 110, obj << 110, obj >> 110)
    compare_both_shifts(obj, 111, obj << 111, obj >> 111)
    compare_both_shifts(obj, 112, obj << 112, obj >> 112)
    compare_both_shifts(obj, 113, obj << 113, obj >> 113)
    compare_both_shifts(obj, 114, obj << 114, obj >> 114)
    compare_both_shifts(obj, 115, obj << 115, obj >> 115)
    compare_both_shifts(obj, 116, obj << 116, obj >> 116)
    compare_both_shifts(obj, 117, obj << 117, obj >> 117)
    compare_both_shifts(obj, 118, obj << 118, obj >> 118)
    compare_both_shifts(obj, 119, obj << 119, obj >> 119)
    compare_both_shifts(obj, 120, obj << 120, obj >> 120)
    compare_both_shifts(obj, 121, obj << 121, obj >> 121)
    compare_both_shifts(obj, 122, obj << 122, obj >> 122)
    compare_both_shifts(obj, 123, obj << 123, obj >> 123)
    compare_both_shifts(obj, 124, obj << 124, obj >> 124)
    compare_both_shifts(obj, 125, obj << 125, obj >> 125)
    compare_both_shifts(obj, 126, obj << 126, obj >> 126)
    compare_both_shifts(obj, 127, obj << 127, obj >> 127)
    compare_both_shifts(obj, 128, obj << 128, obj >> 128)
    compare_both_shifts(obj, 129, obj << 129, obj >> 129)



# mode: run

cdef bint is_null(int* x):
    return False # disabled - currently just a parser test
    match x:
        case NULL:
            return True
        case _:
            return False

def test_is_null():
    """
    >>> test_is_null()
    """
    cdef int some_int = 1
    return  # disabled - currently just a parser test
    assert is_null(&some_int) == False
    assert is_null(NULL) == True



# distutils: language = c++

from Rectangle cimport Rectangle

cdef class PyRectangle:
    cdef Rectangle c_rect

    def __init__(self, int x0, int y0, int x1, int y1):
        self.c_rect = Rectangle(x0, y0, x1, y1)

    def get_area(self):
        return self.c_rect.getArea()

    def get_size(self):
        cdef int width, height
        self.c_rect.getSize(&width, &height)
        return width, height

    def move(self, dx, dy):
        self.c_rect.move(dx, dy)

    # Attribute access
    @property
    def x0(self):
        return self.c_rect.x0
    @x0.setter
    def x0(self, x0):
        self.c_rect.x0 = x0

    # Attribute access
    @property
    def x1(self):
        return self.c_rect.x1
    @x1.setter
    def x1(self, x1):
        self.c_rect.x1 = x1

    # Attribute access
    @property
    def y0(self):
        return self.c_rect.y0
    @y0.setter
    def y0(self, y0):
        self.c_rect.y0 = y0

    # Attribute access
    @property
    def y1(self):
        return self.c_rect.y1
    @y1.setter
    def y1(self, y1):
        self.c_rect.y1 = y1



# mode: run
# tag: cpp, cpp11

from libcpp.limits cimport numeric_limits

cdef extern from *:
    """
    enum class Enum1 {
        Item1,
        Item2
    };
    """
    cdef enum class Enum1:
        Item1
        Item2


cdef extern from * namespace "Namespace1":
    """
    namespace Namespace1 {
        enum class Enum2 {
            Item1,
            Item2
        };
    }
    """
    cdef enum class Enum2:
        Item1
        Item2


cdef enum class Enum3(int):
    a = 1
    b = 2


cdef extern from *:
    """
    enum class sorted
    {
        a = 1,
        b = 0
    };
    """
    cdef enum class Enum4 "sorted":
        a
        b


cdef extern from *:
    """
    #include <limits>

    enum class LongIntEnum : long int {
        val = std::numeric_limits<long int>::max(),
    };
    """
    enum class LongIntEnum(long int):
        val


def test_compare_enums():
    """
    >>> test_compare_enums()
    (True, True, False, False)
    """
    cdef Enum1 x, y
    x = Enum1.Item1
    y = Enum1.Item2

    return (
        x == Enum1.Item1,
        y == Enum1.Item2,
        x == Enum1.Item2,
        y == Enum1.Item1
    )

        
def test_compare_namespace_enums():
    """
    >>> test_compare_enums()
    (True, True, False, False)
    """
    cdef Enum2 z, w
    
    z = Enum2.Item1
    w = Enum2.Item2

    return (
        z == Enum2.Item1,
        w == Enum2.Item2,
        z == Enum2.Item2,
        w == Enum2.Item1
    )


def test_coerce_to_from_py_value(object i):
    """
    >>> test_coerce_to_from_py_value(1)
    (True, False)

    >>> test_coerce_to_from_py_value(2)
    (False, True)

    >>> test_coerce_to_from_py_value(3)
    (False, False)

    >>> test_coerce_to_from_py_value(11111111111111111111111111111111111111111111)
    Traceback (most recent call last):
    OverflowError: Python int too large to convert to C long
    """
    cdef Enum3 x = i
    y = Enum3.b

    return (
        x == Enum3.a,
        y == int(i)
    )


def test_reserved_cname():
    """
    >>> test_reserved_cname()
    True
    """
    cdef Enum4 x = Enum4.a
    return Enum4.a == int(1)


def test_large_enum():
    """
    >>> test_large_enum()
    True
    """
    long_max = int(numeric_limits[long].max())
    return LongIntEnum.val == long_max



def use_volatile():
    cdef volatile int i = 5


cdef const int sum(const int a, const int b):
    return a + b


cdef void pointer_to_const_int(const int *value):
    # Declares value as pointer to const int type.
    # The value can be modified but the object pointed to by value cannot be modified.
    cdef int new_value = 10
    print(value[0])
    value = &new_value
    print(value[0])


cdef void const_pointer_to_int(int * const value):
    # Declares value as const pointer to int type.
    # Value cannot be modified but the object pointed to by value can be modified.
    print(value[0])
    value[0] = 10
    print(value[0])


cdef void const_pointer_to_const_int(const int * const value):
    # Declares value as const pointer to const int type.
    # Neither the value variable nor the int pointed to can be modified.
    print(value[0])



# mode: error

cpdef str a = "123"
cpdef b = 2

cdef class C:
    cpdef float c

def func():
    """
    >>> c = func()
    >>> isinstance(c, C) or c
    True
    """
    cpdef d = C()
    return d


_ERRORS = """
3:0: Variables cannot be declared with 'cpdef'. Use 'cdef' instead.
4:0: Variables cannot be declared with 'cpdef'. Use 'cdef' instead.
7:4: Variables cannot be declared with 'cpdef'. Use 'cdef' instead.
15:4: Variables cannot be declared with 'cpdef'. Use 'cdef' instead.
"""



def use_volatile():
    i: cython.volatile[cython.int] = 5

@cython.cfunc
def sum(a: cython.const[cython.int], b: cython.const[cython.int]) -> cython.const[cython.int]:
    return a + b

@cython.cfunc
def pointer_to_const_int(value: cython.pointer[cython.const[cython.int]]) -> cython.void:
    # Declares value as pointer to const int type (alias: "cython.p_const_int").
    # The value can be modified but the object pointed to by value cannot be modified.
    new_value: cython.int = 10
    print(value[0])
    value = cython.address(new_value)
    print(value[0])

@cython.cfunc
def const_pointer_to_int(value: cython.const[cython.pointer[cython.int]]) -> cython.void:
    # Declares value as const pointer to int type (alias: "cython.const[cython.p_int]").
    # Value cannot be modified but the object pointed to by value can be modified.
    print(value[0])
    value[0] = 10
    print(value[0])

@cython.cfunc
def const_pointer_to_const_int(value: cython.const[cython.pointer[cython.const[cython.int]]]) -> cython.void:
    # Declares value as const pointer to const int type (alias: "cython.const[cython.p_const_int]").
    # Neither the value variable nor the int pointed to can be modified.
    print(value[0])



# mode: run
# tag: cpp, warnings, no-cpp-locals

# This gives a warning about the previous .pxd definition, but should not give an error.
cdef cppclass Foo:
    int _foo
    int get_foo():
        return this._foo
    void set_foo(int foo):
        this._foo = foo

def test_Foo(n):
    """
    >>> test_Foo(1)
    1
    """
    cdef Foo* foo = NULL
    try:
        foo = new Foo()
        foo.set_foo(n)
        return foo.get_foo()
    finally:
        del foo


_WARNINGS = """
5:0: 'Foo' already defined  (ignoring second definition)
"""



# cython: language_level=3
# mode: run
# tag: pep492, pure3.5, gh1462, async, await

import cython

def skip_in_limited_api(f):
    # CYTHON_COMPILING_IN_LIMITED_API is cdef extern in pxd file
    if not cython.compiled or not CYTHON_COMPILING_IN_LIMITED_API:
        return f

# Limited API isn't able to return useful frames for coroutines
@skip_in_limited_api
async def test_coroutine_frame(awaitable):
    """
    >>> class Awaitable(object):
    ...     def __await__(self):
    ...         return iter([2])

    >>> coro = test_coroutine_frame(Awaitable())
    >>> import types
    >>> isinstance(coro.cr_frame, types.FrameType) or coro.cr_frame
    True
    >>> coro.cr_frame is coro.cr_frame  # assert that it's cached
    True
    >>> coro.cr_frame.f_code is not None
    True
    >>> code_obj = coro.cr_frame.f_code
    >>> code_obj.co_argcount
    1
    >>> code_obj.co_varnames
    ('awaitable', 'b')

    >>> next(coro.__await__())  # avoid "not awaited" warning
    2
    """
    b = await awaitable
    return b


# gh1462: Using decorators on coroutines.

def pass_through(func):
    return func


@pass_through
async def test_pass_through():
    """
    >>> t = test_pass_through()
    >>> try: t.send(None)
    ... except StopIteration as ex:
    ...     print(ex.args[0] if ex.args else None)
    ... else: print("NOT STOPPED!")
    None
    """


@pass_through(pass_through)
async def test_pass_through_with_args():
    """
    >>> t = test_pass_through_with_args()
    >>> try: t.send(None)
    ... except StopIteration as ex:
    ...     print(ex.args[0] if ex.args else None)
    ... else: print("NOT STOPPED!")
    None
    """



# mode: run
# ticket: t693

cdef double complex func(double complex x):                                                  
    print "hello"
    return x

def test_coercion():
    """
    >>> c = test_coercion()
    hello
    >>> c.real == 0.5
    True
    >>> c.imag == 1.5
    True
    """
    cdef object x = func(0.5 + 1.5j)
    return x



# mode: run
# cython:infer_types=True

import cython

def divmod_regular(a, b):
    """
    >>> divmod_regular(10, 5)
    (2, 0)
    >>> divmod_regular(9191, 4096)
    (2, 999)
    >>> divmod_regular(10000, 10010)
    (0, 10000)
    >>> divmod_regular(-999999, -111111)
    (9, 0)
    >>> divmod_regular(-888888, -11111)
    (80, -8)
    >>> divmod_regular(-10000, -10086)
    (0, -10000)
    >>> divmod_regular(5, -1)
    (-5, 0)
    >>> divmod_regular(-40, 3)
    (-14, 2)
    >>> divmod_regular(11, -3)
    (-4, -1)
    >>> divmod_regular(-1, 10)
    (-1, 9)
    >>> divmod_regular(0, 9)
    (0, 0)
    >>> divmod_regular(0, -987654321)
    (0, 0)

    >>> divmod_regular(10.0, 5)
    (2.0, 0.0)
    >>> divmod_regular(10, 5.0)
    (2.0, 0.0)
    >>> divmod_regular(10.0, 5.0)
    (2.0, 0.0)
    >>> divmod_regular(9191.0, 4096)
    (2.0, 999.0)
    >>> divmod_regular(9191, 4096.0)
    (2.0, 999.0)
    >>> divmod_regular(9191.0, 4096.0)
    (2.0, 999.0)

    >>> divmod_regular(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_regular(33.0, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_short']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_cshort(a: cython.short, b: cython.short):
    """
    >>> divmod_cshort(10, 5)
    (2, 0)
    >>> divmod_cshort(-10, 5)
    (-2, 0)
    >>> divmod_cshort(10, -5)
    (-2, 0)
    >>> divmod_cshort(-50, 1)
    (-50, 0)
    >>> divmod_cshort(-40, 3)
    (-14, 2)
    >>> divmod_cshort(11, -3)
    (-4, -1)
    >>> divmod_cshort(-1, 10)
    (-1, 9)
    >>> divmod_cshort(0, 9)
    (0, 0)

    >>> divmod_cshort(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_int']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_cint(a: cython.int, b: cython.int):
    """
    >>> divmod_cint(10, 5)
    (2, 0)
    >>> divmod_cint(-10, 5)
    (-2, 0)
    >>> divmod_cint(10, -5)
    (-2, 0)
    >>> divmod_cint(9191, 4096)
    (2, 999)
    >>> divmod_cint(10000, 10010)
    (0, 10000)
    >>> divmod_cint(-999999, -111111)
    (9, 0)
    >>> divmod_cint(-888888, -11111)
    (80, -8)
    >>> divmod_cint(-10000, -10086)
    (0, -10000)
    >>> divmod_cint(-50, 1)
    (-50, 0)
    >>> divmod_cint(-40, 3)
    (-14, 2)
    >>> divmod_cint(11, -3)
    (-4, -1)
    >>> divmod_cint(-1, 10)
    (-1, 9)
    >>> divmod_cint(0, 9)
    (0, 0)
    >>> divmod_cint(0, -987654321)
    (0, 0)

    >>> divmod_cint(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_long']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_clong(a: cython.long, b: cython.long):
    """
    >>> divmod_clong(10, 5)
    (2, 0)
    >>> divmod_clong(-10, 5)
    (-2, 0)
    >>> divmod_clong(10, -5)
    (-2, 0)
    >>> divmod_clong(9191, 4096)
    (2, 999)
    >>> divmod_clong(10000, 10010)
    (0, 10000)
    >>> divmod_clong(-999999, -111111)
    (9, 0)
    >>> divmod_clong(-888888, -11111)
    (80, -8)
    >>> divmod_clong(-10000, -10086)
    (0, -10000)
    >>> divmod_clong(-50, 1)
    (-50, 0)
    >>> divmod_clong(-40, 3)
    (-14, 2)
    >>> divmod_clong(11, -3)
    (-4, -1)
    >>> divmod_clong(-1, 10)
    (-1, 9)
    >>> divmod_clong(0, 9)
    (0, 0)
    >>> divmod_clong(0, -987654321)
    (0, 0)

    >>> divmod_clong(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_PY_LONG_LONG']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_clonglong(a: cython.longlong, b: cython.longlong):
    """
    >>> divmod_clonglong(10, 5)
    (2, 0)
    >>> divmod_clonglong(-10, 5)
    (-2, 0)
    >>> divmod_clonglong(10, -5)
    (-2, 0)
    >>> divmod_clonglong(9191, 4096)
    (2, 999)
    >>> divmod_clonglong(10000, 10010)
    (0, 10000)
    >>> divmod_clonglong(88321773, 98539211)
    (0, 88321773)
    >>> divmod_clonglong(-99999999, -11111111)
    (9, 0)
    >>> divmod_clonglong(-88888888, -1111111)
    (80, -8)
    >>> divmod_clonglong(-88837244, -110119120)
    (0, -88837244)
    >>> divmod_clonglong(-5000000, 1)
    (-5000000, 0)
    >>> divmod_clonglong(-100190, 17)
    (-5894, 8)
    >>> divmod_clonglong(2014014349, -19)
    (-106000756, -15)
    >>> divmod_clonglong(0, 996007985)
    (0, 0)
    >>> divmod_clonglong(-10000, -10086)
    (0, -10000)
    >>> divmod_clonglong(-50, 1)
    (-50, 0)
    >>> divmod_clonglong(-40, 3)
    (-14, 2)
    >>> divmod_clonglong(11, -3)
    (-4, -1)
    >>> divmod_clonglong(-1, 10)
    (-1, 9)
    >>> divmod_clonglong(0, 9)
    (0, 0)
    >>> divmod_clonglong(0, -987654321)
    (0, 0)

    >>> divmod_clonglong(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_clonglong(0, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_PY_LONG_LONG']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_mixed_cint_clonglong(a: cython.int, b: cython.longlong):
    """
    >>> divmod_mixed_cint_clonglong(10, 5)
    (2, 0)
    >>> divmod_mixed_cint_clonglong(9191, 4096)
    (2, 999)

    >>> divmod_mixed_cint_clonglong(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_mixed_cint_clonglong(0, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_PY_LONG_LONG']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_mixed_clonglong_cint(a: cython.longlong, b: cython.int):
    """
    >>> divmod_mixed_clonglong_cint(10, 5)
    (2, 0)
    >>> divmod_mixed_clonglong_cint(9191, 4096)
    (2, 999)
    >>> divmod_mixed_clonglong_cint(19283090123, 1230912)
    (15665, 853643)
    >>> divmod_mixed_clonglong_cint(-420000000000, 1000)
    (-420000000, 0)

    >>> divmod_mixed_clonglong_cint(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_mixed_clonglong_cint(0, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_PY_LONG_LONG']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_mixed_unpack(a: cython.longlong, b: cython.int):
    """
    >>> divmod_mixed_unpack(10, 5)
    (2, 0)
    >>> divmod_mixed_unpack(9191, 4096)
    (2, 999)
    >>> divmod_mixed_unpack(19283090123, 1230912)
    (15665, 853643)

    >>> divmod_mixed_unpack(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_mixed_unpack(0, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    q, r = divmod(a, b)
    if cython.compiled:
        assert cython.typeof(q) == 'long long', cython.typeof(q)
        assert cython.typeof(r) == 'long long', cython.typeof(r)
    return q, r


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_PY_LONG_LONG']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_mixed_const(a: cython.longlong):
    """
    >>> divmod_mixed_const(-420000000000)
    (-420000000, 0)
    >>> divmod_mixed_const(9191)
    (9, 191)
    >>> divmod_mixed_const(19283090123)
    (19283090, 123)
    """
    q, r = divmod(a, 1000)
    if cython.compiled:
        assert cython.typeof(q) == 'long long', cython.typeof(q)
        assert cython.typeof(r) == 'long long', cython.typeof(r)
    return q, r


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_float_double']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_cfloat(a: cython.float, b: cython.float):
    """
    >>> divmod_cfloat(10.0, 5.0)
    (2.0, 0.0)
    >>> divmod_cfloat(-10.0, 5.0)
    (-2.0, 0.0)
    >>> divmod_cfloat(10.0, -5.0)
    (-2.0, -0.0)
    >>> divmod_cfloat(9191.0, 4096.0)
    (2.0, 999.0)
    >>> divmod_cfloat(10000.0, 10010.0)
    (0.0, 10000.0)
    >>> divmod_cfloat(-999999.0, -111111.0)
    (9.0, -0.0)
    >>> divmod_cfloat(-888888.0, -11111.0)
    (80.0, -8.0)
    >>> divmod_cfloat(-10000.0, -10086.0)
    (0.0, -10000.0)
    >>> divmod_cfloat(-50.0, 1.0)
    (-50.0, 0.0)
    >>> divmod_cfloat(-40.0, 3.0)
    (-14.0, 2.0)
    >>> divmod_cfloat(11.0, -3.0)
    (-4.0, -1.0)
    >>> divmod_cfloat(-1.0, 10.0)
    (-1.0, 9.0)
    >>> divmod_cfloat(0.0, 9.0)
    (0.0, 0.0)
    >>> divmod_cfloat(0.0, -987654321.0)
    (-0.0, -0.0)

    >>> divmod_cfloat(33.0, 0.0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_float_double']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_cdouble(a: cython.double, b: cython.double):
    """
    >>> divmod_cdouble(10.0, 5.0)
    (2.0, 0.0)
    >>> divmod_cdouble(-10.0, 5.0)
    (-2.0, 0.0)
    >>> divmod_cdouble(10.0, -5.0)
    (-2.0, -0.0)
    >>> divmod_cdouble(9191.0, 4096.0)
    (2.0, 999.0)
    >>> divmod_cdouble(10000.0, 10010.0)
    (0.0, 10000.0)
    >>> divmod_cdouble(-999999.0, -111111.0)
    (9.0, -0.0)
    >>> divmod_cdouble(-888888.0, -11111.0)
    (80.0, -8.0)
    >>> divmod_cdouble(-10000.0, -10086.0)
    (0.0, -10000.0)
    >>> divmod_cdouble(-50.0, 1.0)
    (-50.0, 0.0)
    >>> divmod_cdouble(-40.0, 3.0)
    (-14.0, 2.0)
    >>> divmod_cdouble(11.0, -3.0)
    (-4.0, -1.0)
    >>> divmod_cdouble(-1.0, 10.0)
    (-1.0, 9.0)
    >>> divmod_cdouble(0.0, 9.0)
    (0.0, 0.0)
    >>> divmod_cdouble(0.0, -987654321.0)
    (-0.0, -0.0)

    >>> divmod_cdouble(33.0, 0.0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_float_double']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_clongdouble(a: cython.longdouble, b: cython.double):
    """
    >>> divmod_clongdouble(10.0, 5.0)
    (2.0, 0.0)
    >>> divmod_clongdouble(-10.0, 5.0)
    (-2.0, 0.0)
    >>> divmod_clongdouble(10.0, -5.0)
    (-2.0, -0.0)
    >>> divmod_clongdouble(9191.0, 4096.0)
    (2.0, 999.0)
    >>> divmod_clongdouble(10000.0, 10010.0)
    (0.0, 10000.0)
    >>> divmod_clongdouble(-999999.0, -111111.0)
    (9.0, -0.0)
    >>> divmod_clongdouble(-888888.0, -11111.0)
    (80.0, -8.0)
    >>> divmod_clongdouble(-10000.0, -10086.0)
    (0.0, -10000.0)
    >>> divmod_clongdouble(-50.0, 1.0)
    (-50.0, 0.0)
    >>> divmod_clongdouble(-40.0, 3.0)
    (-14.0, 2.0)
    >>> divmod_clongdouble(11.0, -3.0)
    (-4.0, -1.0)
    >>> divmod_clongdouble(-1.0, 10.0)
    (-1.0, 9.0)
    >>> divmod_clongdouble(0.0, 9.0)
    (0.0, 0.0)
    >>> divmod_clongdouble(0.0, -987654321.0)
    (-0.0, -0.0)

    >>> divmod_clongdouble(33.0, 0.0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_float_double']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_mixed_cdouble(a: cython.double, b: cython.float):
    """
    >>> divmod_mixed_cdouble(10.0, 5.0)
    (2.0, 0.0)
    >>> divmod_mixed_cdouble(-10.0, 5.0)
    (-2.0, 0.0)
    >>> divmod_mixed_cdouble(10.0, -5.0)
    (-2.0, -0.0)
    >>> divmod_mixed_cdouble(9191.0, 4096.0)
    (2.0, 999.0)
    >>> divmod_mixed_cdouble(10000.0, 10010.0)
    (0.0, 10000.0)
    >>> divmod_mixed_cdouble(-999999.0, -111111.0)
    (9.0, -0.0)
    >>> divmod_mixed_cdouble(-888888.0, -11111.0)
    (80.0, -8.0)
    >>> divmod_mixed_cdouble(-10000.0, -10086.0)
    (0.0, -10000.0)
    >>> divmod_mixed_cdouble(-50.0, 1.0)
    (-50.0, 0.0)
    >>> divmod_mixed_cdouble(-40.0, 3.0)
    (-14.0, 2.0)
    >>> divmod_mixed_cdouble(11.0, -3.0)
    (-4.0, -1.0)
    >>> divmod_mixed_cdouble(-1.0, 10.0)
    (-1.0, 9.0)
    >>> divmod_mixed_cdouble(0.0, 9.0)
    (0.0, 0.0)
    >>> divmod_mixed_cdouble(0.0, -987654321.0)
    (-0.0, -0.0)

    >>> divmod_mixed_cdouble(33.0, 0.0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_nogil_int_long']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_int_nogil(a: cython.long, b: cython.int):
    """
    >>> divmod_int_nogil(10, 5)
    (2, 0)
    >>> divmod_int_nogil(-10, 5)
    (-2, 0)
    >>> divmod_int_nogil(9191, 4096)
    (2, 999)

    >>> divmod_int_nogil(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_int_nogil(0, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    with cython.nogil:
        q, r = divmod(a, b)
    return q, r


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_nogil_float_double']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_float_nogil(a: cython.float, b: cython.double):
    """
    >>> divmod_float_nogil(10.0, 5.0)
    (2.0, 0.0)
    >>> divmod_float_nogil(-10.0, 5.0)
    (-2.0, 0.0)
    >>> divmod_float_nogil(9191.0, 4096.0)
    (2.0, 999.0)

    >>> divmod_float_nogil(33.0, 0.0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_float_nogil(0.0, 0.0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    with cython.nogil:
        q, r = divmod(a, b)
    return q, r



# mode: error
# tag: or, binop, warnings

def test_or(a, b):
    return a || b


_WARNINGS = """
5:13: Found the C operator '||', did you mean the Python operator 'or'?
"""

_ERRORS = """
5:13: Syntax error in simple statement list
"""



# cython: c_string_encoding=default
# cython: c_string_type=str

# NOTE: the directive order above is specifically meant to trigger (and confuse) the
# source encoding detector with "coding=default".

__doc__ = r"""
    >>> as_objects("ab\xff") == "ab\xff"
    True
    >>> slice_as_objects("ab\xffd", 1, 4) == "b\xff"
    True
"""

include "str_ascii_auto_encoding.pyx"



"""
Benchmark recursive async generators implemented in python
by traversing a binary tree.

Author: Kumar Aditya
"""

from __future__ import annotations

import asyncio
import time
from collections.abc import AsyncIterator


class Tree:
    def __init__(self, left: Tree | None, value: int, right: Tree | None) -> None:
        self.left = left
        self.value = value
        self.right = right

    async def __aiter__(self) -> AsyncIterator[int]:
        if self.left:
            async for i in self.left:
                yield i
        yield self.value
        if self.right:
            async for i in self.right:
                yield i


def tree(input: range) -> Tree | None:
    n = len(input)
    if n == 0:
        return None
    i = n // 2
    return Tree(tree(input[:i]), input[i], tree(input[i + 1:]))


async def bench_async_generators(async_tree) -> None:
    async for _ in async_tree:
        pass



def run_benchmark(repeat=10, scale: cython.long = 1, timer=time.perf_counter):
    s: cython.long

    async_tree = tree(range(1000))

    timings = []
    for _ in range(repeat):
        t = timer()
        for s in range(scale):
            asyncio.run(bench_async_generators(async_tree))
        t = timer() - t
        timings.append(t)
    return timings




def list_insert(list l):
    """
    >>> list_insert([1,2,3])
    [1, 4, 2, 3]
    """
    list.insert(l, 1, 4)
    return l


def list_insert_literal():
    """
    >>> list_insert_literal()
    (None, [1, 4, 2, 3])
    """
    l = [1,2,3]
    r = list.insert(l, 1, 4)
    return r, l


def list_insert_assigned():
    """
    >>> list_insert_assigned()
    (None, [1, 4, 2, 3])
    """
    insert = list.insert
    l = [1,2,3]
    r = insert(l, 1, 4)
    return r, l


def list_pop():
    """
    >>> list_pop()
    (2, [1, 3])
    """
    l = [1,2,3]
    r = list.pop(l, 1)
    return r, l


def list_pop_assigned():
    """
    >>> list_pop_assigned()
    [1, 3]
    """
    pop = list.pop
    l = [1,2,3]
    pop(l, 1)
    return l



cdef sorteditems(d):
    return tuple(sorted(d.items()))


def spam(x, y, z):
    """
    >>> spam(1,2,3)
    (1, 2, 3)
    >>> spam(1,2)
    Traceback (most recent call last):
    TypeError: spam() takes exactly 3 positional arguments (2 given)
    >>> spam(1,2,3,4)
    Traceback (most recent call last):
    TypeError: spam() takes exactly 3 positional arguments (4 given)
    >>> spam(1,2,3, a=1)
    Traceback (most recent call last):
    TypeError: spam() got an unexpected keyword argument 'a'
    """
    return (x, y, z)

def grail(x, y, z, *a):
    """
    >>> grail(1,2,3)
    (1, 2, 3, ())
    >>> grail(1,2,3,4)
    (1, 2, 3, (4,))
    >>> grail(1,2,3,4,5,6,7,8,9)
    (1, 2, 3, (4, 5, 6, 7, 8, 9))
    >>> grail(1,2)
    Traceback (most recent call last):
    TypeError: grail() takes at least 3 positional arguments (2 given)
    >>> grail(1,2,3, a=1)
    Traceback (most recent call last):
    TypeError: grail() got an unexpected keyword argument 'a'
    """
    return (x, y, z, a)

def swallow(x, y, z, **k):
    """
    >>> swallow(1,2,3)
    (1, 2, 3, ())
    >>> swallow(1,2,3,4)
    Traceback (most recent call last):
    TypeError: swallow() takes exactly 3 positional arguments (4 given)
    >>> swallow(1,2,3, a=1, b=2)
    (1, 2, 3, (('a', 1), ('b', 2)))
    >>> swallow(1,2,3, x=1)
    Traceback (most recent call last):
    TypeError: swallow() got multiple values for keyword argument 'x'
    """
    return (x, y, z, sorteditems(k))

def creosote(x, y, z, *a, **k):
    """
    >>> creosote(1,2,3)
    (1, 2, 3, (), ())
    >>> creosote(1,2,3,4)
    (1, 2, 3, (4,), ())
    >>> creosote(1,2,3, a=1)
    (1, 2, 3, (), (('a', 1),))
    >>> creosote(1,2,3,4, a=1, b=2)
    (1, 2, 3, (4,), (('a', 1), ('b', 2)))
    >>> creosote(1,2,3,4, x=1)
    Traceback (most recent call last):
    TypeError: creosote() got multiple values for keyword argument 'x'
    """
    return (x, y, z, a, sorteditems(k))

def onlyt(*a):
    """
    >>> onlyt(1)
    (1,)
    >>> onlyt(1,2)
    (1, 2)
    >>> onlyt(a=1)
    Traceback (most recent call last):
    TypeError: onlyt() got an unexpected keyword argument 'a'
    >>> onlyt(1, a=2)
    Traceback (most recent call last):
    TypeError: onlyt() got an unexpected keyword argument 'a'
    >>> test_no_copy_args(onlyt)
    True
    """
    return a

def onlyk(**k):
    """
    >>> onlyk(a=1)
    (('a', 1),)
    >>> onlyk(a=1, b=2)
    (('a', 1), ('b', 2))
    >>> onlyk(1)
    Traceback (most recent call last):
    TypeError: onlyk() takes exactly 0 positional arguments (1 given)
    >>> onlyk(1, 2)
    Traceback (most recent call last):
    TypeError: onlyk() takes exactly 0 positional arguments (2 given)
    >>> onlyk(1, a=1, b=2)
    Traceback (most recent call last):
    TypeError: onlyk() takes exactly 0 positional arguments (1 given)
    """
    return sorteditems(k)

def tk(*a, **k):
    """
    >>> tk(a=1)
    (('a', 1),)
    >>> tk(a=1, b=2)
    (('a', 1), ('b', 2))
    >>> tk(1)
    (1,)
    >>> tk(1, 2)
    (1, 2)
    >>> tk(1, a=1, b=2)
    (1, ('a', 1), ('b', 2))
    """
    return a + sorteditems(k)

def t_kwonly(*a, k):
    """
    >>> test_no_copy_args(t_kwonly, k=None)
    True
    """
    return a


def test_no_copy_args(func, **kw):
    """
    func is a function such that func(*args, **kw) returns args.
    We test that no copy is made of the args tuple.
    This tests both the caller side and the callee side.
    """
    args = (1, 2, 3)
    return func(*args, **kw) is args



# mode: run
# ticket: t698
# distutils: extra_compile_args=-fwrapv

cdef extern from *:
    int INT_MAX
    long LONG_MAX

max_int = INT_MAX
max_long = LONG_MAX
max_long_long = (<object>2) ** (sizeof(long long) * 8 - 1) - 1


cimport cython

def abs_as_name():
    """
    >>> _abs = abs_as_name()
    >>> _abs(-5)
    5
    """
    x = abs
    return x

def py_abs(a):
    """
    >>> py_abs(-5)
    5
    >>> py_abs(-5.5)
    5.5
    """
    return abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = 'abs']")
def sub_abs(int a):
    """
    >>> sub_abs(5)
    (-5, 95)
    >>> sub_abs(105)
    (-105, -5)
    """
    return -abs(a), 100 - abs(a)

@cython.overflowcheck(True)
@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = 'abs']")
def int_abs(int a):
    """
    >>> int_abs(-5) == 5
    True
    >>> int_abs(-5.1) == 5
    True
    >>> int_abs(-max_int-1)     #doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    OverflowError: ...
    >>> int_abs(max_int) == abs(max_int)         or (max_int, int_abs(max_int), abs(max_int))
    True
    """
    return abs(a)

@cython.overflowcheck(True)
@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = 'abs']")
cdef int c_int_abs(int a) except * nogil:
    return abs(a)

def test_c_int_abs(int a):
    """
    >>> test_c_int_abs(-5) == 5
    True
    >>> test_c_int_abs(-5.1) == 5
    True
    >>> test_c_int_abs(-max_int-1)     #doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    OverflowError: ...
    >>> test_c_int_abs(max_int) == abs(max_int)  or (max_int, test_c_int_abs(max_int), abs(max_int))
    True
    """
    return c_int_abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']")
@cython.test_fail_if_path_exists("//ReturnStatNode//NameNode[@entry.cname = 'abs']",
                                 "//ReturnStatNode//NameNode[@entry.cname = 'labs']")
def uint_abs(unsigned int a):
    """
    >>> uint_abs(max_int) == abs(max_int)         or (max_int, uint_abs(max_int), abs(max_int))
    True
    """
    return abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']")
@cython.test_fail_if_path_exists("//ReturnStatNode//NameNode[@entry.cname = 'abs']",
                                 "//ReturnStatNode//NameNode[@entry.cname = 'labs']")
cdef unsigned int c_uint_abs(unsigned int a) nogil:
    return abs(a)

def test_c_uint_abs(unsigned int a):
    """
    >>> test_c_uint_abs(max_int) == abs(max_int)  or (max_int, test_c_uint_abs(max_int), abs(max_int))
    True
    """
    return c_uint_abs(a)

@cython.overflowcheck(True)
@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = 'labs']")
def long_abs(long a):
    """
    >>> long_abs(-5) == 5
    True
    >>> long_abs(-5.1) == 5
    True
    >>> long_abs(-max_long-1)     #doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    OverflowError: ...
    >>> long_abs(max_long) == abs(max_long)         or (max_long, long_abs(max_long), abs(max_long))
    True
    """
    return abs(a)

@cython.overflowcheck(True)
@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = 'labs']")
cdef long c_long_abs(long a) except * nogil:
    return abs(a)

def test_c_long_abs(long a):
    """
    >>> test_c_long_abs(-5) == 5
    True
    >>> test_c_long_abs(-5.1) == 5
    True
    >>> test_c_long_abs(-max_long-1)     #doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    OverflowError: ...
    >>> test_c_long_abs(max_long) == abs(max_long)  or (max_long, test_c_long_abs(max_long), abs(max_long))
    True
    """
    return c_long_abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']")
@cython.test_fail_if_path_exists("//ReturnStatNode//NameNode[@entry.cname = 'abs']",
                                 "//ReturnStatNode//NameNode[@entry.cname = 'labs']")
def ulong_abs(unsigned long a):
    """
    >>> ulong_abs(max_long) == abs(max_long)         or (max_int, ulong_abs(max_long), abs(max_long))
    True
    >>> ulong_abs(max_long + 5) == abs(max_long + 5)         or (max_long + 5, ulong_abs(max_long + 5), abs(max_long + 5))
    True
    """
    return abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']")
@cython.test_fail_if_path_exists("//ReturnStatNode//NameNode[@entry.cname = 'abs']",
                                 "//ReturnStatNode//NameNode[@entry.cname = 'labs']")
cdef unsigned long c_ulong_abs(unsigned long a) nogil:
    return abs(a)

def test_c_ulong_abs(unsigned long a):
    """
    >>> test_c_ulong_abs(max_long) == abs(max_long)  or (max_int, test_c_ulong_abs(max_long), abs(max_long))
    True
    >>> test_c_ulong_abs(max_long + 5) == abs(max_long + 5)  or (max_long + 5, test_c_ulong_abs(max_long + 5), abs(max_long + 5))
    True
    """
    return c_ulong_abs(a)

@cython.overflowcheck(True)
@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = '__Pyx_abs_longlong']")
def long_long_abs(long long a):
    """
    >>> long_long_abs(-(2**33)) == 2**33
    True
    >>> long_long_abs(-max_long_long-1)     #doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    OverflowError: ...
    >>> long_long_abs(max_long_long) == abs(max_long_long)        or (max_long_long, long_long_abs(max_long_long), abs(max_long_long))
    True
    """
    return abs(a)

@cython.overflowcheck(True)
@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = '__Pyx_abs_longlong']")
cdef long long c_long_long_abs(long long a) except * nogil:
    return abs(a)

def test_c_long_long_abs(long long a):
    """
    >>> test_c_long_long_abs(-(2**33)) == 2**33
    True
    >>> test_c_long_long_abs(-max_long_long-1)     #doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    OverflowError: ...
    >>> test_c_long_long_abs(max_long_long) == abs(max_long_long) or (max_long_long, test_c_long_long_abs(max_long_long), abs(max_long_long))
    True
    """
    return c_long_long_abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = 'fabs']")
def double_abs(double a):
    """
    >>> double_abs(-5)
    5.0
    >>> double_abs(-5.5)
    5.5
    """
    return abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = 'fabs']")
cdef double c_double_abs(double a) nogil:
    return abs(a)

def test_c_double_abs(double a):
    """
    >>> test_c_double_abs(-5)
    5.0
    >>> test_c_double_abs(-5.5)
    5.5
    """
    return c_double_abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = 'fabsf']")
def float_abs(float a):
    """
    >>> float_abs(-5)
    5.0
    >>> float_abs(-5.5)
    5.5
    """
    return abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = 'fabsf']")
cdef float c_float_abs(float a) nogil:
    return abs(a)

def test_c_float_abs(float a):
    """
    >>> test_c_float_abs(-5)
    5.0
    >>> test_c_float_abs(-5.5)
    5.5
    """
    return c_float_abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = '__Pyx_c_abs_double']")
def complex_abs(complex a):
    """
    >>> complex_abs(-5j)
    5.0
    >>> complex_abs(-5.5j)
    5.5
    """
    return abs(a)

@cython.test_assert_path_exists("//ReturnStatNode//NameNode[@entry.name = 'abs']",
                                "//ReturnStatNode//NameNode[@entry.cname = '__Pyx_c_abs_double']")
cdef double c_complex_abs(complex a) nogil:
    return abs(a)

def test_c_complex_abs(complex a):
    """
    >>> test_c_complex_abs(-5j)
    5.0
    >>> test_c_complex_abs(-5.5j)
    5.5
    """
    return c_complex_abs(a)



def primes(kmax):
    p = []
    k = 0
    n = 2
    while k < kmax:
        i = 0
        while i < k and n % p[i] != 0:
            i += 1
        if i == k:
            p.append(n)
            k += 1
        n += 1
    return p



# tag: run

# This file is also included from "parallel.pyx" to run the same tests
# but with OpenMP enabled.

cimport cython.parallel
from cython.parallel import prange, threadid
from cython.view cimport array
from libc.stdlib cimport malloc, calloc, free, abort
from libc.stdio cimport puts

import os
import sys
import sysconfig

try:
    from builtins import next # Py3k
except ImportError:
    def next(it):
        return it.next()

def skip_in_freethreading(f):
    # defined in parallel.pyx
    if "OPENMP_PARALLEL" in globals():
        if sysconfig.get_config_var("Py_GIL_DISABLED"):
            return None
    return f

#@cython.test_assert_path_exists(
#    "//ParallelWithBlockNode//ParallelRangeNode[@schedule = 'dynamic']",
#    "//GILStatNode[@state = 'nogil]//ParallelRangeNode")
def test_prange():
    """
    >>> test_prange()
    (9, 9, 45, 45)
    """
    cdef Py_ssize_t i, j, sum1 = 0, sum2 = 0

    with nogil, cython.parallel.parallel():
        for i in prange(10, schedule='dynamic'):
            sum1 += i

    for j in prange(10, nogil=True):
        sum2 += j

    return i, j, sum1, sum2

def test_descending_prange():
    """
    >>> test_descending_prange()
    5
    """
    cdef int i, start = 5, stop = -5, step = -2
    cdef int sum = 0

    for i in prange(start, stop, step, nogil=True):
        sum += i

    return sum

def test_prange_matches_range(int start, int stop, int step):
    """
    >>> test_prange_matches_range(0, 8, 3)
    >>> test_prange_matches_range(0, 9, 3)
    >>> test_prange_matches_range(0, 10, 3)

    >>> test_prange_matches_range(0, 10, -3)

    >>> test_prange_matches_range(0, -10, -3)
    >>> test_prange_matches_range(1, -10, -3)
    >>> test_prange_matches_range(2, -10, -3)
    >>> test_prange_matches_range(3, -10, -3)
    """
    cdef int i = -765432, range_last = -876543, prange_last = -987654
    prange_set = set()
    for i in prange(start, stop, step, nogil=True, num_threads=3):
        prange_last = i
        with gil:
            prange_set.add(i)
    range_set = set(range(start, stop, step))
    assert range_set == prange_set, "missing: %s extra %s" % (sorted(range_set-prange_set), sorted(prange_set - range_set))
    for ii in range(start, stop, step):
        range_last = ii
    if range_set:
        assert prange_last == i
        assert range_last == prange_last


def test_propagation():
    """
    >>> test_propagation()
    (9, 9, 9, 9, 450, 450)
    """
    cdef int i = 0, j = 0, x = 0, y = 0
    cdef int sum1 = 0, sum2 = 0

    for i in prange(10, nogil=True):
        for j in prange(10):
            sum1 += i

    with nogil, cython.parallel.parallel():
        for x in prange(10):
            for y in prange(10):
                sum2 += y

    return i, j, x, y, sum1, sum2

# DISABLED, not allowed in OpenMP 3.0 (fails on Windows)
#def test_unsigned_operands():
#    """
#    >>> test_unsigned_operands()
#    10
#    """
#    cdef int i
#    cdef int start = -5
#    cdef unsigned int stop = 5
#    cdef int step = 1
#
#    cdef int steps_taken = 0
#    cdef int *steps_takenp = &steps_taken
#
#    for i in prange(start, stop, step, nogil=True):
#        steps_taken += 1
#        if steps_takenp[0] > 10:
#            abort()
#
#    return steps_taken

def test_reassign_start_stop_step():
    """
    >>> test_reassign_start_stop_step()
    20
    """
    cdef int start = 0, stop = 10, step = 2
    cdef int i
    cdef int sum = 0

    for i in prange(start, stop, step, nogil=True):
        start = -2
        stop = 2
        step = 0

        sum += i

    return sum

def test_closure_parallel_privates():
    """
    >>> test_closure_parallel_privates()
    9 9
    45 45
    0 0 9 9
    """
    cdef int x

    def test_target():
        nonlocal x
        for x in prange(10, nogil=True):
            pass
        return x

    print test_target(), x

    def test_reduction():
        nonlocal x
        cdef int i

        x = 0
        for i in prange(10, nogil=True):
            x += i

        return x

    print test_reduction(), x

    def test_generator():
        nonlocal x
        cdef int i

        x = 0
        yield x
        x = 2

        for i in prange(10, nogil=True):
            x = i

        yield x

    g = test_generator()
    print next(g), x, next(g), x

def test_closure_parallel_with_gil():
    """
    >>> test_closure_parallel_with_gil()
    45
    45
    """
    cdef int sum = 0
    temp1 = 5
    temp2 = -5

    def test_reduction():
        nonlocal sum, temp1, temp2

        cdef int i

        for i in prange(10, nogil=True):
            with gil:
                sum += temp1 + temp2 + i
                # assert abs(sum - sum) == 0

        return sum

    print test_reduction()
    print sum

def test_pure_mode():
    """
    >>> test_pure_mode()
    0
    1
    2
    3
    4
    4
    3
    2
    1
    0
    0
    """
    import Cython.Shadow
    pure_parallel = sys.modules['cython.parallel']

    for i in pure_parallel.prange(5):
        print i

    for i in pure_parallel.prange(4, -1, -1, schedule='dynamic', nogil=True):
        print i

    with pure_parallel.parallel():
        print pure_parallel.threadid()

cdef extern from "types.h":
    ctypedef short actually_long_t
    ctypedef long actually_short_t

ctypedef int myint_t

def test_nan_init():
    """
    >>> test_nan_init()
    """
    cdef int mybool = 0
    cdef int err = 0
    cdef int *errp = &err

    cdef signed char a1 = 10
    cdef unsigned char a2 = 10
    cdef short b1 = 10
    cdef unsigned short b2 = 10
    cdef int c1 = 10
    cdef unsigned int c2 = 10
    cdef long d1 = 10
    cdef unsigned long d2 = 10
    cdef long long e1 = 10
    cdef unsigned long long e2 = 10

    cdef actually_long_t miss1 = 10
    cdef actually_short_t miss2 = 10
    cdef myint_t typedef1 = 10

    cdef float f = 10.0
    cdef double g = 10.0
    cdef long double h = 10.0

    cdef void *p = <void *> 10

    with nogil, cython.parallel.parallel():
        # First, trick the error checking to make it believe these variables
        # are initialized after this if

        if mybool: # mybool is always false!
            a1 = a2 = b1 = b2 = c1 = c2 = d1 = d2 = e1 = e2 = 0
            f = g = h = 0.0
            p = NULL
            miss1 = miss2 = typedef1 = 0

        if (a1 == 10 or a2 == 10 or
            b1 == 10 or b2 == 10 or
            c1 == 10 or c2 == 10 or
            d1 == 10 or d2 == 10 or
            e1 == 10 or e2 == 10 or
            f == 10.0 or g == 10.0 or h == 10.0 or
            p == <void *> 10 or miss1 == 10 or miss2 == 10
            or typedef1 == 10):
            errp[0] = 1

    cdef int i
    for i in prange(10, nogil=True):
        # First, trick the error checking to make it believe these variables
        # are initialized after this if

        if mybool: # mybool is always false!
            a1 = a2 = b1 = b2 = c1 = c2 = d1 = d2 = e1 = e2 = 0
            f = g = h = 0.0
            p = NULL
            miss1 = miss2 = typedef1 = 0

        if (a1 == 10 or a2 == 10 or
            b1 == 10 or b2 == 10 or
            c1 == 10 or c2 == 10 or
            d1 == 10 or d2 == 10 or
            e1 == 10 or e2 == 10 or
            f == 10.0 or g == 10.0 or h == 10.0 or
            p == <void *> 10 or miss1 == 10 or miss2 == 10
            or typedef1 == 10):
            errp[0] = 1

    if err:
        raise Exception("One of the values was not initialized to a maximum "
                        "or NaN value")

    c1 = 20
    with nogil, cython.parallel.parallel():
        c1 = 16


cdef void nogil_print(char *s) noexcept with gil:
    print s.decode('ascii')

def test_else_clause():
    """
    >>> test_else_clause()
    else clause executed
    """
    cdef int i

    for i in prange(5, nogil=True):
        pass
    else:
        nogil_print('else clause executed')

def test_prange_break():
    """
    >>> test_prange_break()
    """
    cdef int i

    for i in prange(10, nogil=True):
        if i == 8:
            break
    else:
        nogil_print('else clause executed')

def test_prange_continue():
    """
    >>> test_prange_continue()
    else clause executed
    0 0
    1 0
    2 2
    3 0
    4 4
    5 0
    6 6
    7 0
    8 8
    9 0
    """
    cdef int i
    cdef int *p = <int *> calloc(10, sizeof(int))

    if p == NULL:
        raise MemoryError

    for i in prange(10, nogil=True):
        if i % 2 != 0:
            continue

        p[i] = i
    else:
        nogil_print('else clause executed')

    for i in range(10):
       print i, p[i]

    free(p)

def test_nested_break_continue():
    """
    >>> test_nested_break_continue()
    6 7 6 7
    8
    """
    cdef int i, j, result1 = 0, result2 = 0

    for i in prange(10, nogil=True, num_threads=2, schedule='static'):
        for j in prange(10, num_threads=2, schedule='static'):
            if i == 6 and j == 7:
                result1 = i
                result2 = j
                break
        else:
            continue

        break

    print i, j, result1, result2

    with nogil, cython.parallel.parallel(num_threads=2):
        for i in prange(10, schedule='static'):
            if i == 8:
                break
            else:
                continue

    print i

cdef int parallel_return() noexcept nogil:
    cdef int i

    for i in prange(10):
        if i == 8:
            return i
    else:
        return 1

    return 2

def test_return():
    """
    >>> test_return()
    8
    """
    print parallel_return()

def test_parallel_exceptions():
    """
    >>> test_parallel_exceptions()
    I am executed first
    ('propagate me',) 0
    """
    cdef int i, j, sum = 0

    mylist = []

    try:
        for i in prange(10, nogil=True):
            try:
                for j in prange(10):
                    with gil:
                        raise Exception("propagate me")

                    sum += i * j
                sum += i
            finally:
                with gil:
                    mylist.append("I am executed first")
    except Exception, e:
        print mylist[0]
        print e.args, sum

def test_parallel_exceptions_unnested():
    """
    >>> test_parallel_exceptions_unnested()
    ('I am executed first', 0)
    ('propagate me',) 0
    """
    cdef int i, sum = 0

    mylist = []

    try:
        with nogil, cython.parallel.parallel():
            try:
                for i in prange(10):
                    with gil:
                        raise Exception("propagate me")

                    sum += i
            finally:
                with gil:
                    mylist.append(("I am executed first", sum))
    except Exception, e:
        print mylist[0]
        print e.args, sum

cdef int parallel_exc_cdef() except -3:
    cdef int i, j
    for i in prange(10, nogil=True):
        for j in prange(10, num_threads=6):
            with gil:
                raise Exception("propagate me")

    return 0

cdef int parallel_exc_cdef_unnested() except -3:
    cdef int i
    for i in prange(10, nogil=True):
        with gil:
            raise Exception("propagate me")

    return 0

def test_parallel_exc_cdef():
    """
    >>> test_parallel_exc_cdef()
    Traceback (most recent call last):
        ...
    Exception: propagate me
    """
    parallel_exc_cdef_unnested()
    parallel_exc_cdef()

cpdef int parallel_exc_cpdef() except -3:
    cdef int i, j
    for i in prange(10, nogil=True):
        for j in prange(10, num_threads=6):
            with gil:
                raise Exception("propagate me")

    return 0

cpdef int parallel_exc_cpdef_unnested() except -3:
    cdef int i, j
    for i in prange(10, nogil=True):
        with gil:
            raise Exception("propagate me")

    return 0


def test_parallel_exc_cpdef():
    """
    >>> test_parallel_exc_cpdef()
    Traceback (most recent call last):
        ...
    Exception: propagate me
    """
    parallel_exc_cpdef_unnested()
    parallel_exc_cpdef()

cdef int parallel_exc_nogil_swallow() except -1:
    cdef int i, j
    for i in prange(10, nogil=True):
        try:
            for j in prange(10):
                with gil:
                    raise Exception("propagate me")
        finally:
            return i

    return 0

cdef int parallel_exc_nogil_swallow_unnested() except -1:
    cdef int i
    with nogil:
        try:
            for i in prange(10):
                with gil:
                    raise Exception("propagate me")
        finally:
            return i

    return 0

def test_parallel_exc_nogil_swallow():
    """
    >>> test_parallel_exc_nogil_swallow()
    execute me
    execute me
    """
    parallel_exc_nogil_swallow_unnested()
    print 'execute me'
    parallel_exc_nogil_swallow()
    print 'execute me'

def parallel_exc_replace():
    """
    >>> parallel_exc_replace()
    Traceback (most recent call last):
        ...
    Exception: propagate me instead
    """
    cdef int i, j
    for i in prange(10, nogil=True):
        with gil:
            try:
                for j in prange(10, nogil=True):
                    with gil:
                        raise Exception("propagate me")
            except Exception, e:
                raise Exception("propagate me instead")

    return 0


def parallel_exceptions2():
    """
    >>> parallel_exceptions2()
    Traceback (most recent call last):
       ...
    Exception: propagate me
    """
    cdef int i, j, k

    for i in prange(10, nogil=True):
        for j in prange(10):
            for k in prange(10):
                if i + j + k > 20:
                    with gil:
                        raise Exception("propagate me")
                        break
                    continue
                    return

@skip_in_freethreading  # Reassignment of 'obj'
def test_parallel_with_gil_return():
    """
    >>> test_parallel_with_gil_return()
    True
    45
    """
    cdef int i, sum = 0

    for i in prange(10, nogil=True):
        with gil:
            obj = i
            sum += obj

    print obj in range(10)

    with nogil, cython.parallel.parallel():
        with gil:
            return sum

def test_parallel_with_gil_continue_unnested():
    """
    >>> test_parallel_with_gil_continue_unnested()
    20
    """
    cdef int i, sum = 0

    for i in prange(10, nogil=True):
        with gil:
            if i % 2:
                continue

        sum += i

    print sum


cdef int inner_parallel_section() noexcept nogil:
    cdef int j, sum = 0
    for j in prange(10):
        sum += j
    return sum

def outer_parallel_section():
    """
    >>> outer_parallel_section()
    450
    """
    cdef int i, sum = 0
    for i in prange(10, nogil=True):
        sum += inner_parallel_section()
    return sum

cdef int nogil_cdef_except_clause() except -1 nogil:
    return 1

cdef void nogil_cdef_except_star() except * nogil:
    pass

def test_nogil_cdef_except_clause():
    """
    >>> test_nogil_cdef_except_clause()
    """
    cdef int i
    for i in prange(10, nogil=True):
        nogil_cdef_except_clause()
        nogil_cdef_except_star()

def test_num_threads_compile():
    cdef int i
    for i in prange(10, nogil=True, num_threads=2):
        pass

    with nogil, cython.parallel.parallel(num_threads=2):
        pass

    with nogil, cython.parallel.parallel(num_threads=2):
        for i in prange(10):
            pass

cdef int chunksize() noexcept nogil:
    return 3

def test_chunksize():
    """
    >>> test_chunksize()
    45
    45
    45
    """
    cdef int i, sum

    sum = 0
    for i in prange(10, nogil=True, num_threads=2, schedule='static', chunksize=chunksize()):
        sum += i
    print sum

    sum = 0
    for i in prange(10, nogil=True, num_threads=6, schedule='dynamic', chunksize=chunksize()):
        sum += i
    print sum

    sum = 0
    with nogil, cython.parallel.parallel():
        for i in prange(10, schedule='guided', chunksize=chunksize()):
            sum += i
    print sum


cdef class PrintOnDealloc(object):
    def __dealloc__(self):
        print "deallocating..."

def error():
    raise Exception("propagate me")

def test_clean_temps():
    """
    >>> test_clean_temps()
    deallocating...
    propagate me
    """
    cdef Py_ssize_t i

    try:
        for i in prange(100, nogil=True, num_threads=1):
            with gil:
                # freethreading - if we actually assigned to x this would currently be dodgy
                # but the test doesn't get that far.
                x = PrintOnDealloc() + error()
    except Exception, e:
        print e.args[0]


def test_pointer_temps(double x):
    """
    >>> test_pointer_temps(1.0)
    4.0
    """
    cdef Py_ssize_t i
    cdef double* f
    cdef double[:] arr = array(format="d", shape=(10,), itemsize=sizeof(double))
    arr[0] = 4.0
    arr[1] = 3.0

    for i in prange(10, nogil=True, num_threads=1):
        f = &arr[0]

    return f[0]


@skip_in_freethreading  #  "+=" on a Python object isn't atomic
def test_prange_in_with(int x, ctx):
    """
    >>> from contextlib import contextmanager
    >>> @contextmanager
    ... def ctx(l): yield l
    >>> test_prange_in_with(4, ctx([0]))
    6
    """
    cdef int i
    with ctx as l:
        for i in prange(x, nogil=True):
            with gil:
                l[0] += i
        return l[0]


cdef extern from *:
    """
    #ifdef _OPENMP
    #define _get_addr(_x, _idx) &_x
    #else
    #define _get_addr(_x, _idx) (&_x+_idx)
    #endif
    #define address_of_temp(store, temp, idx) store = _get_addr(temp, idx)
    #define address_of_temp2(store, ignore, temp, idx) store = _get_addr(temp, idx)

    double get_value() {
        return 1.0;
    }
    """
    void address_of_temp(...) nogil
    void address_of_temp2(...) nogil
    double get_value() except -1.0 nogil  # will generate a temp for exception checking

def test_inner_private():
    """
    Determines if a temp variable is private by taking its address in multiple threads
    and seeing if they're the same (thread private variables should have different
    addresses
    >>> test_inner_private()
    ok
    """
    cdef double* not_parallel[2]
    cdef double* inner_vals[2]
    cdef double* outer_vals[2]
    cdef Py_ssize_t n, m

    for n in range(2):
        address_of_temp(not_parallel[n], get_value(), 0)
    assert not_parallel[0] == not_parallel[1], "Addresses should be the same since they come from the same temp"

    for n in prange(2, num_threads=2, schedule='static', chunksize=1, nogil=True):
        address_of_temp(outer_vals[n], get_value(), n)
        for m in prange(1):
            # second temp just ensures different numbering
            address_of_temp2(inner_vals[n], get_value(), get_value(), n)

    inner_are_the_same = inner_vals[0] == inner_vals[1]
    outer_are_the_same = outer_vals[0] == outer_vals[1]

    assert outer_are_the_same == False, "Temporary variables in outer loop should be private"
    assert inner_are_the_same == False,  "Temporary variables in inner loop should be private"

    print('ok')

cdef void prange_exception_checked_function(int* ptr, int id) except * nogil:
    # requires the GIL after each call
    ptr[0] = id;

cdef void prange_call_exception_checked_function_impl(int* arr, int N) nogil:
    # Inside a nogil function, prange can't be sure the GIL has been released.
    # Therefore Cython must release the GIL itself.
    # Otherwise, we can experience cause lock-ups if anything inside it acquires the GIL
    # (since if any other thread has finished, it will be holding the GIL).
    #
    # An equivalent test with prange is in "sequential_parallel.pyx"
    cdef int i
    for i in prange(N, num_threads=4, schedule='static', chunksize=1):
        prange_exception_checked_function(arr+i, i)

def test_prange_call_exception_checked_function():
    """
    >>> test_prange_call_exception_checked_function()
    """

    cdef int N = 10000
    cdef int* buf = <int*>malloc(sizeof(int)*N)
    if buf == NULL:
        raise MemoryError
    try:
        # Don't release the GIL
        prange_call_exception_checked_function_impl(buf, N)

        for i in range(N):
            assert buf[i] == i
    finally:
        free(buf)

def test_prange_type_inference_1(short end):
    """
    >>> test_prange_type_inference_1(1)
    """
    for i in prange(end, nogil=True):
        pass
    assert cython.typeof(i) == "short", cython.typeof(i)

def test_prange_type_inference_2(long start, short end):
    """
    >>> test_prange_type_inference_2(-100, 5)
    """
    for i in prange(start, end, nogil=True):
        pass
    assert cython.typeof(i) == "long", cython.typeof(i)

def test_prange_type_inference_3(short start, short end, short step):
    """
    >>> test_prange_type_inference_3(-100, 5, 2)
    """
    for i in prange(start, end, step, nogil=True):
        pass
    # Addition in "step" makes it expand the type to "int"
    assert cython.typeof(i) == "int", cython.typeof(i)



import random
from libc.stdlib cimport malloc, free

def random_noise(int number=1):
    cdef int i
    # allocate number * sizeof(double) bytes of memory
    cdef double *my_array = <double *> malloc(
        number * sizeof(double))
    if not my_array:
        raise MemoryError()

    try:
        ran = random.normalvariate
        for i in range(number):
            my_array[i] = ran(0, 1)

        # ... let's just assume we do some more heavy C calculations here to make up
        # for the work that it takes to pack the C double values into Python float
        # objects below, right after throwing away the existing objects above.

        return [x for x in my_array[:number]]
    finally:
        # return the previously allocated memory to the system
        free(my_array)



# Defines the standard C++ cast operators.
#
# Due to type restrictions, these are only defined for pointer parameters,
# however that is the only case where they are significantly more interesting
# than the standard C cast operator which can be written "<T>(expression)" in
# Cython.

cdef extern from * nogil:
    cdef T dynamic_cast[T](void *) except +   # nullptr may also indicate failure
    cdef T static_cast[T](void *)
    cdef T reinterpret_cast[T](void *)
    cdef T const_cast[T](void *)



# mode: error

cdef signed   float       e
cdef unsigned float       f
cdef signed   double      g
cdef unsigned double      h
cdef signed   long double i
cdef unsigned long double j


_ERRORS = u"""
3:5: Unrecognised type modifier combination
4:5: Unrecognised type modifier combination
5:5: Unrecognised type modifier combination
6:5: Unrecognised type modifier combination
7:5: Unrecognised type modifier combination
8:5: Unrecognised type modifier combination
"""



# mode: error
from cython.cimports.cython import view

def main():
    # VALID
    a: cython.int[::view.indirect, ::1, :]
    b: cython.int[::view.indirect, :, ::1]
    c: cython.int[::view.indirect_contiguous, ::1, :]

    # INVALID
    d: cython.int[::view.contiguous, ::view.indirect, :]
    e: cython.int[::1, ::view.indirect, :]

_ERRORS = u"""
12:17: Only dimension 2 may be contiguous and direct
"""




class test(object):
    a = 1
t = test()

def getattr2_literal_unicode(a):
    """
    >>> getattr2_literal_unicode(t)
    1
    >>> getattr2_literal_unicode(object())
    Traceback (most recent call last):
    AttributeError: 'object' object has no attribute 'a'
    """
    return getattr(a, u"a")

def getattr3_literal_unicode(a, b):
    """
    >>> getattr3_literal_unicode(t, 2)
    (1, 2)
    """
    return getattr(a, u"a", b), getattr(a, u"b", b)

def getattr2_simple(a, b):
    """
    >>> getattr2_simple(t, 'a')
    1
    >>> getattr2_simple(t, 'b')
    Traceback (most recent call last):
    AttributeError: 'test' object has no attribute 'b'
    """
    return getattr(a, b)

def getattr3_explicit(a, b, c):
    """
    >>> getattr3_explicit(t, 'a', 2)
    1
    >>> getattr3_explicit(t, 'b', 2)
    2
    """
    return getattr3(a, b, c)

def getattr3_args(a, b, c):
    """
    >>> getattr3_args(t, 'a', 2)
    1
    >>> getattr3_args(t, 'b', 2)
    2
    """
    return getattr(a, b, c)
